@@ -39,6 +39,11 @@
 
 #include "config.h"
 
+#ifdef HAVE_FONTS
+#include <cairo.h>
+#include <pango/pangocairo.h>
+#endif
+
 #include "clutter/clutter-backend-private.h"
 #include "clutter/clutter-context-private.h"
 #include "clutter/clutter-debug.h"
@@ -82,8 +87,6 @@ clutter_backend_dispose (GObject *gobject)
   /* clear the events still in the queue of the main context */
   _clutter_clear_events_queue ();
 
-  g_clear_object (&backend->cogl_display);
-  g_clear_object (&backend->cogl_context);
   g_clear_object (&backend->dummy_onscreen);
   if (backend->stage_window)
     {
@@ -93,8 +96,9 @@ clutter_backend_dispose (GObject *gobject)
     }
 
   g_clear_pointer (&backend->cogl_source, g_source_destroy);
-  g_clear_pointer (&backend->font_name, g_free);
+#ifdef HAVE_FONTS
   g_clear_pointer (&backend->font_options, cairo_font_options_destroy);
+#endif
   g_clear_object (&backend->input_method);
 
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
@@ -138,6 +142,7 @@ clutter_backend_set_property (GObject      *object,
     }
 }
 
+#ifdef HAVE_FONTS
 static void
 clutter_backend_real_resolution_changed (ClutterBackend *backend)
 {
@@ -154,12 +159,14 @@ clutter_backend_real_resolution_changed (ClutterBackend *backend)
     resolution = dpi / 1024.0;
 
   if (context->font_map != NULL)
-    cogl_pango_font_map_set_resolution (context->font_map, resolution);
+    pango_cairo_font_map_set_resolution (PANGO_CAIRO_FONT_MAP (context->font_map),
+                                         resolution);
 }
+#endif
 
 static gboolean
 clutter_backend_do_real_create_context (ClutterBackend  *backend,
-                                        CoglDriver       driver_id,
+                                        CoglDriverId     driver_id,
                                         GError         **error)
 {
   ClutterBackendClass *klass;
@@ -208,11 +215,11 @@ error:
 static const struct {
   const char *driver_name;
   const char *driver_desc;
-  CoglDriver driver_id;
+  CoglDriverId driver_id;
 } all_known_drivers[] = {
-  { "gl3", "OpenGL 3.1 core profile", COGL_DRIVER_GL3 },
-  { "gles2", "OpenGL ES 2.0", COGL_DRIVER_GLES2 },
-  { "any", "Default Cogl driver", COGL_DRIVER_ANY },
+  { "gl3", "OpenGL 3.1 core profile", COGL_DRIVER_ID_GL3 },
+  { "gles2", "OpenGL ES 2.0", COGL_DRIVER_ID_GLES2 },
+  { "any", "Default Cogl driver", COGL_DRIVER_ID_ANY },
 };
 
 static gboolean
@@ -342,7 +349,9 @@ clutter_backend_class_init (ClutterBackendClass *klass)
 
   g_object_class_install_properties (gobject_class, N_PROPS, pspecs);
 
+#ifdef HAVE_FONTS
   klass->resolution_changed = clutter_backend_real_resolution_changed;
+#endif
 
   klass->create_context = clutter_backend_real_create_context;
 }
@@ -353,6 +362,19 @@ clutter_backend_init (ClutterBackend *self)
   self->dummy_onscreen = NULL;
 
   self->fallback_resource_scale = 1.f;
+
+  /* Default font options */
+#ifdef HAVE_FONTS
+  self->font_options = cairo_font_options_create ();
+  cairo_font_options_set_hint_metrics (self->font_options,
+                                       CAIRO_HINT_METRICS_ON);
+  cairo_font_options_set_hint_style (self->font_options,
+                                     CAIRO_HINT_STYLE_NONE);
+  cairo_font_options_set_subpixel_order (self->font_options,
+                                         CAIRO_SUBPIXEL_ORDER_DEFAULT);
+  cairo_font_options_set_antialias (self->font_options,
+                                    CAIRO_ANTIALIAS_DEFAULT);
+#endif
 }
 
 ClutterStageWindow *
@@ -450,70 +472,6 @@ clutter_backend_get_resolution (ClutterBackend *backend)
   return resolution / 1024.0;
 }
 
-/**
- * clutter_backend_set_font_options:
- * @backend: a #ClutterBackend
- * @options: Cairo font options for the backend, or %NULL
- *
- * Sets the new font options for @backend. The #ClutterBackend will
- * copy the #cairo_font_options_t.
- *
- * If @options is %NULL, the first following call to
- * [method@Clutter.Backend.get_font_options] will return the default font
- * options for @backend.
- *
- * This function is intended for actors creating a Pango layout
- * using the PangoCairo API.
- */
-void
-clutter_backend_set_font_options (ClutterBackend             *backend,
-                                  const cairo_font_options_t *options)
-{
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-
-  if (backend->font_options != options)
-    {
-      if (backend->font_options)
-        cairo_font_options_destroy (backend->font_options);
-
-      if (options)
-        backend->font_options = cairo_font_options_copy (options);
-      else
-        backend->font_options = NULL;
-
-      g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
-    }
-}
-
-/**
- * clutter_backend_get_font_options:
- * @backend: a #ClutterBackend
- *
- * Retrieves the font options for @backend.
- *
- * Return value: (transfer none): the font options of the #ClutterBackend.
- *   The returned #cairo_font_options_t is owned by the backend and should
- *   not be modified or freed
- */
-const cairo_font_options_t *
-clutter_backend_get_font_options (ClutterBackend *backend)
-{
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
-
-  if (G_LIKELY (backend->font_options))
-    return backend->font_options;
-
-  backend->font_options = cairo_font_options_create ();
-
-  cairo_font_options_set_hint_style (backend->font_options, CAIRO_HINT_STYLE_NONE);
-  cairo_font_options_set_subpixel_order (backend->font_options, CAIRO_SUBPIXEL_ORDER_DEFAULT);
-  cairo_font_options_set_antialias (backend->font_options, CAIRO_ANTIALIAS_DEFAULT);
-
-  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
-
-  return backend->font_options;
-}
-
 /**
  * clutter_backend_get_cogl_context:
  * @backend: a #ClutterBackend
