@@ -235,19 +235,6 @@ typedef enum
   CLUTTER_TEXT_DIRECTION_RTL
 } ClutterTextDirection;
 
-/**
- * ClutterShaderType:
- * @CLUTTER_VERTEX_SHADER: a vertex shader
- * @CLUTTER_FRAGMENT_SHADER: a fragment shader
- *
- * The type of GLSL shader program
- */
-typedef enum
-{
-  CLUTTER_VERTEX_SHADER,
-  CLUTTER_FRAGMENT_SHADER
-} ClutterShaderType;
-
 /**
  * ClutterModifierType:
  * @CLUTTER_SHIFT_MASK: Mask applied by the Shift key
@@ -519,6 +506,7 @@ typedef enum /*< flags prefix=CLUTTER_EVENT >*/
   CLUTTER_EVENT_FLAG_RELATIVE_MOTION = 1 << 3,
   CLUTTER_EVENT_FLAG_GRAB_NOTIFY  = 1 << 4,
   CLUTTER_EVENT_FLAG_POINTER_EMULATED = 1 << 5,
+  CLUTTER_EVENT_FLAG_A11Y_MODIFIER_FIRST_CLICK = 1 << 6,
 } ClutterEventFlags;
 
 /**
@@ -849,7 +837,7 @@ typedef enum
  * @CLUTTER_REPAINT_FLAGS_POST_PAINT: Run the repaint function after
  *   painting the stages
  *
- * Flags to pass to clutter_threads_add_repaint_func_full().
+ * Flags to pass to [func@Clutter.threads_add_repaint_func].
  */
 typedef enum
 {
@@ -957,38 +945,6 @@ typedef enum
   CLUTTER_REPEAT_BOTH   = CLUTTER_REPEAT_X_AXIS | CLUTTER_REPEAT_Y_AXIS
 } ClutterContentRepeat;
 
-/**
- * ClutterColorspace:
- * @CLUTTER_COLORSPACE_DEFAULT: Unknown colorspace
- * @CLUTTER_COLORSPACE_SRGB: Default sRGB colorspace
- * @CLUTTER_COLORSPACE_BT2020: BT2020 colorspace
- *
- * Colorspace information.
- */
-typedef enum
-{
-  CLUTTER_COLORSPACE_DEFAULT,
-  CLUTTER_COLORSPACE_SRGB,
-  CLUTTER_COLORSPACE_BT2020,
-} ClutterColorspace;
-
-/**
- * ClutterTransferFunction:
- * @CLUTTER_TRANSFER_FUNCTION_DEFAULT: Unknown transfer function
- * @CLUTTER_TRANSFER_FUNCTION_SRGB: The sRGB gamma transfer function
- * @CLUTTER_TRANSFER_FUNCTION_PQ: Perceptual quantizer (PQ)
- * @CLUTTER_TRANSFER_FUNCTION_LINEAR: Linear transfer function
- *
- * Colorspace information.
- */
-typedef enum
-{
-  CLUTTER_TRANSFER_FUNCTION_DEFAULT,
-  CLUTTER_TRANSFER_FUNCTION_SRGB,
-  CLUTTER_TRANSFER_FUNCTION_PQ,
-  CLUTTER_TRANSFER_FUNCTION_LINEAR,
-} ClutterTransferFunction;
-
 /**
  * ClutterEncodingRequiredFormat:
  * @CLUTTER_ENCODING_REQUIRED_FORMAT_UINT8: 8bpc uint
