@@ -48,8 +48,7 @@
  * clutter_create_context: (skip)
  */
 CLUTTER_EXPORT
-ClutterContext * clutter_create_context (ClutterContextFlags         flags,
-                                         ClutterBackendConstructor   backend_constructor,
+ClutterContext * clutter_create_context (ClutterBackendConstructor   backend_constructor,
                                          gpointer                    user_data,
                                          GError                    **error);
 
@@ -106,14 +105,6 @@ gboolean clutter_stage_get_device_coords (ClutterStage         *stage,
                                           ClutterInputDevice   *device,
                                           ClutterEventSequence *sequence,
                                           graphene_point_t     *coords);
-CLUTTER_EXPORT
-void clutter_stage_repick_device (ClutterStage       *stage,
-                                  ClutterInputDevice *device);
-
-CLUTTER_EXPORT
-void clutter_get_debug_flags (ClutterDebugFlag     *debug_flags,
-                              ClutterDrawDebugFlag *draw_flags,
-                              ClutterPickDebugFlag *pick_flags);
 
 CLUTTER_EXPORT
 void clutter_actor_notify_transform_invalid (ClutterActor *self);
