@@ -17,6 +17,10 @@
 
 #include "config.h"
 
+#ifdef HAVE_FONTS
+#include <cairo/cairo.h>
+#endif
+
 #include "clutter/clutter-settings.h"
 
 #include "clutter/clutter-context-private.h"
@@ -30,18 +34,6 @@
 
 #define DEFAULT_FONT_NAME       "Sans 12"
 
-typedef struct
-{
-  cairo_antialias_t cairo_antialias;
-  gint clutter_font_antialias;
-
-  cairo_hint_style_t cairo_hint_style;
-  const char *clutter_font_hint_style;
-
-  cairo_subpixel_order_t cairo_subpixel_order;
-  const char *clutter_font_subpixel_order;
-} FontSettings;
-
 struct _ClutterSettings
 {
   GObject parent_instance;
@@ -61,18 +53,9 @@ struct _ClutterSettings
   gchar *font_name;
   gint font_dpi;
 
-  gint xft_hinting;
-  gint xft_antialias;
-  gchar *xft_hint_style;
-  gchar *xft_rgba;
-
   gint long_press_duration;
 
-  guint last_fontconfig_timestamp;
-
   guint password_hint_time;
-
-  gint unscaled_font_dpi;
 };
 
 enum
@@ -86,17 +69,12 @@ enum
 
   PROP_FONT_NAME,
 
-  PROP_FONT_ANTIALIAS,
   PROP_FONT_DPI,
-  PROP_FONT_HINTING,
-  PROP_FONT_HINT_STYLE,
-  PROP_FONT_RGBA,
 
   PROP_LONG_PRESS_DURATION,
 
   PROP_PASSWORD_HINT_TIME,
 
-  PROP_UNSCALED_FONT_DPI,
 
   PROP_LAST
 };
@@ -105,79 +83,6 @@ static GParamSpec *obj_props[PROP_LAST];
 
 G_DEFINE_FINAL_TYPE (ClutterSettings, clutter_settings, G_TYPE_OBJECT);
 
-static inline void
-settings_update_font_options (ClutterSettings *self)
-{
-  cairo_hint_style_t hint_style = CAIRO_HINT_STYLE_NONE;
-  cairo_antialias_t antialias_mode = CAIRO_ANTIALIAS_GRAY;
-  cairo_subpixel_order_t subpixel_order = CAIRO_SUBPIXEL_ORDER_DEFAULT;
-  cairo_font_options_t *options;
-
-  if (self->backend == NULL)
-    return;
-
-  options = cairo_font_options_create ();
-
-  cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_ON);
-
-  if (self->xft_hinting >= 0 &&
-      self->xft_hint_style == NULL)
-    {
-      hint_style = CAIRO_HINT_STYLE_NONE;
-    }
-  else if (self->xft_hint_style != NULL)
-    {
-      if (strcmp (self->xft_hint_style, "hintnone") == 0)
-        hint_style = CAIRO_HINT_STYLE_NONE;
-      else if (strcmp (self->xft_hint_style, "hintslight") == 0)
-        hint_style = CAIRO_HINT_STYLE_SLIGHT;
-      else if (strcmp (self->xft_hint_style, "hintmedium") == 0)
-        hint_style = CAIRO_HINT_STYLE_MEDIUM;
-      else if (strcmp (self->xft_hint_style, "hintfull") == 0)
-        hint_style = CAIRO_HINT_STYLE_FULL;
-    }
-
-  cairo_font_options_set_hint_style (options, hint_style);
-
-  if (self->xft_rgba)
-    {
-      if (strcmp (self->xft_rgba, "rgb") == 0)
-        subpixel_order = CAIRO_SUBPIXEL_ORDER_RGB;
-      else if (strcmp (self->xft_rgba, "bgr") == 0)
-        subpixel_order = CAIRO_SUBPIXEL_ORDER_BGR;
-      else if (strcmp (self->xft_rgba, "vrgb") == 0)
-        subpixel_order = CAIRO_SUBPIXEL_ORDER_VRGB;
-      else if (strcmp (self->xft_rgba, "vbgr") == 0)
-        subpixel_order = CAIRO_SUBPIXEL_ORDER_VBGR;
-    }
-
-  cairo_font_options_set_subpixel_order (options, subpixel_order);
-
-  if (self->xft_antialias >= 0 && !self->xft_antialias)
-    antialias_mode = CAIRO_ANTIALIAS_NONE;
-  else if (subpixel_order != CAIRO_SUBPIXEL_ORDER_DEFAULT)
-    antialias_mode = CAIRO_ANTIALIAS_SUBPIXEL;
-  else if (self->xft_antialias >= 0)
-    antialias_mode = CAIRO_ANTIALIAS_GRAY;
-
-  cairo_font_options_set_antialias (options, antialias_mode);
-
-  CLUTTER_NOTE (BACKEND, "New font options:\n"
-                " - font-name:  %s\n"
-                " - antialias:  %d\n"
-                " - hinting:    %d\n"
-                " - hint-style: %s\n"
-                " - rgba:       %s\n",
-                self->font_name != NULL ? self->font_name : DEFAULT_FONT_NAME,
-                self->xft_antialias,
-                self->xft_hinting,
-                self->xft_hint_style != NULL ? self->xft_hint_style : "<null>",
-                self->xft_rgba != NULL ? self->xft_rgba : "<null>");
-
-  clutter_backend_set_font_options (self->backend, options);
-  cairo_font_options_destroy (options);
-}
-
 static void
 settings_update_font_name (ClutterSettings *self)
 {
@@ -205,18 +110,26 @@ settings_update_resolution (ClutterSettings *self)
         self->resolution *= scale;
     }
 
-  CLUTTER_NOTE (BACKEND, "New resolution: %.2f (%s)",
-                self->resolution,
-                self->unscaled_font_dpi > 0 ? "unscaled" : "scaled");
+  CLUTTER_NOTE (BACKEND, "New resolution: %.2f",
+                self->resolution);
 
   if (self->backend != NULL)
     g_signal_emit_by_name (self->backend, "resolution-changed");
 }
 
+#ifdef HAVE_FONTS
 static void
-get_font_gsettings (GSettings    *settings,
-                    FontSettings *output)
+clutter_settings_update_font_options (ClutterSettings *self)
 {
+  GSettings *settings = self->font_settings;
+  cairo_hint_style_t hint_style = CAIRO_HINT_STYLE_NONE;
+  cairo_antialias_t antialias_mode = CAIRO_ANTIALIAS_GRAY;
+  cairo_subpixel_order_t subpixel_order = CAIRO_SUBPIXEL_ORDER_DEFAULT;
+  const char *clutter_font_hint_style = NULL, *clutter_font_subpixel_order = NULL;
+
+  if (self->backend == NULL)
+    return;
+
   /* org.gnome.desktop.GDesktopFontAntialiasingMode */
   static const struct
   {
@@ -263,60 +176,43 @@ get_font_gsettings (GSettings    *settings,
   i = g_settings_get_enum (settings, "font-hinting");
   if (i < G_N_ELEMENTS (hintings))
     {
-      output->cairo_hint_style = hintings[i].cairo_hint_style;
-      output->clutter_font_hint_style = hintings[i].clutter_font_hint_style;
-    }
-  else
-    {
-      output->cairo_hint_style = CAIRO_HINT_STYLE_DEFAULT;
-      output->clutter_font_hint_style = NULL;
-    }
-
-  i = g_settings_get_enum (settings, "font-antialiasing");
-  if (i < G_N_ELEMENTS (antialiasings))
-    {
-      output->cairo_antialias = antialiasings[i].cairo_antialias;
-      output->clutter_font_antialias = antialiasings[i].clutter_font_antialias;
-    }
-  else
-    {
-      output->cairo_antialias = CAIRO_ANTIALIAS_DEFAULT;
-      output->clutter_font_antialias = -1;
+      hint_style = hintings[i].cairo_hint_style;
+      clutter_font_hint_style = hintings[i].clutter_font_hint_style;
     }
+  cairo_font_options_set_hint_style (self->backend->font_options, hint_style);
 
   i = g_settings_get_enum (settings, "font-rgba-order");
   if (i < G_N_ELEMENTS (rgba_orders))
     {
-      output->cairo_subpixel_order = rgba_orders[i].cairo_subpixel_order;
-      output->clutter_font_subpixel_order = rgba_orders[i].clutter_font_subpixel_order;
-    }
-  else
-    {
-      output->cairo_subpixel_order = CAIRO_SUBPIXEL_ORDER_DEFAULT;
-      output->clutter_font_subpixel_order = NULL;
+      subpixel_order = rgba_orders[i].cairo_subpixel_order;
+      clutter_font_subpixel_order = rgba_orders[i].clutter_font_subpixel_order;
     }
+  cairo_font_options_set_subpixel_order (self->backend->font_options, subpixel_order);
 
-  if (output->cairo_antialias == CAIRO_ANTIALIAS_GRAY)
-    output->clutter_font_subpixel_order = "none";
-}
+  i = g_settings_get_enum (settings, "font-antialiasing");
+  if (i < G_N_ELEMENTS (antialiasings))
+    antialias_mode = antialiasings[i].cairo_antialias;
 
-static void
-init_font_options (ClutterSettings *self)
-{
-  GSettings *settings = self->font_settings;
-  cairo_font_options_t *options = cairo_font_options_create ();
-  FontSettings fs;
+  if (subpixel_order == CAIRO_SUBPIXEL_ORDER_DEFAULT)
+    antialias_mode = CAIRO_ANTIALIAS_SUBPIXEL;
 
-  get_font_gsettings (settings, &fs);
+  cairo_font_options_set_antialias (self->backend->font_options, antialias_mode);
 
-  cairo_font_options_set_hint_style (options, fs.cairo_hint_style);
-  cairo_font_options_set_antialias (options, fs.cairo_antialias);
-  cairo_font_options_set_subpixel_order (options, fs.cairo_subpixel_order);
+  CLUTTER_NOTE (BACKEND, "New font options:\n"
+                " - font-name:  %s\n"
+                " - antialias:  %d\n"
+                " - hinting:    %d\n"
+                " - hint-style: %s\n"
+                " - rgba:       %s\n",
+                self->font_name != NULL ? self->font_name : DEFAULT_FONT_NAME,
+                antialias_mode,
+                hint_style == CAIRO_HINT_STYLE_NONE ? 0 : 1,
+                clutter_font_hint_style != NULL ? clutter_font_hint_style : "<null>",
+                clutter_font_subpixel_order != NULL ? clutter_font_subpixel_order : "<null>");
 
-  clutter_backend_set_font_options (self->backend, options);
-
-  cairo_font_options_destroy (options);
+  g_signal_emit_by_name (self->backend, "font-changed");
 }
+#endif
 
 static void
 sync_mouse_options (ClutterSettings *self)
@@ -333,27 +229,20 @@ sync_mouse_options (ClutterSettings *self)
                 NULL);
 }
 
+#ifdef HAVE_FONTS
 static gboolean
 on_font_settings_change_event (GSettings *settings,
-			       gpointer   keys,
-			       gint       n_keys,
-			       gpointer   user_data)
+                               gpointer   keys,
+                               gint       n_keys,
+                               gpointer   user_data)
 {
   ClutterSettings *self = CLUTTER_SETTINGS (user_data);
-  FontSettings fs;
-  gint hinting;
 
-  get_font_gsettings (settings, &fs);
-  hinting = fs.cairo_hint_style == CAIRO_HINT_STYLE_NONE ? 0 : 1;
-  g_object_set (self,
-                "font-hinting",        hinting,
-                "font-hint-style",     fs.clutter_font_hint_style,
-                "font-antialias",      fs.clutter_font_antialias,
-                "font-subpixel-order", fs.clutter_font_subpixel_order,
-                NULL);
+  clutter_settings_update_font_options (self);
 
   return FALSE;
 }
+#endif
 
 static gboolean
 on_mouse_settings_change_event (GSettings *settings,
@@ -470,12 +359,15 @@ on_mouse_a11y_settings_change_event (GSettings *settings,
 static void
 load_initial_settings (ClutterSettings *self)
 {
+#ifdef HAVE_FONTS
   static const gchar *font_settings_path = "org.gnome.desktop.interface";
+#endif
   static const gchar *mouse_settings_path = "org.gnome.desktop.peripherals.mouse";
   static const char *mouse_a11y_settings_path = "org.gnome.desktop.a11y.mouse";
   GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
-  g_autoptr (GSettingsSchema) schema = NULL;
+  GSettingsSchema *schema;
 
+#ifdef HAVE_FONTS
   schema = g_settings_schema_source_lookup (source, font_settings_path, TRUE);
   if (!schema)
     {
@@ -486,14 +378,13 @@ load_initial_settings (ClutterSettings *self)
       self->font_settings = g_settings_new_full (schema, NULL, NULL);
       if (self->font_settings)
         {
-          init_font_options (self);
+          clutter_settings_update_font_options (self);
           g_signal_connect (self->font_settings, "change-event",
                             G_CALLBACK (on_font_settings_change_event),
                             self);
         }
-
-      g_clear_pointer (&schema, g_settings_schema_unref);
     }
+ #endif
 
   schema = g_settings_schema_source_lookup (source, mouse_settings_path, TRUE);
   if (!schema)
@@ -510,8 +401,6 @@ load_initial_settings (ClutterSettings *self)
                             G_CALLBACK (on_mouse_settings_change_event),
                             self);
         }
-
-      g_clear_pointer (&schema, g_settings_schema_unref);
     }
 
   schema = g_settings_schema_source_lookup (source, mouse_a11y_settings_path, TRUE);
@@ -534,8 +423,6 @@ clutter_settings_finalize (GObject *gobject)
   ClutterSettings *self = CLUTTER_SETTINGS (gobject);
 
   g_free (self->font_name);
-  g_free (self->xft_hint_style);
-  g_free (self->xft_rgba);
 
   g_clear_object (&self->font_settings);
   g_clear_object (&self->mouse_settings);
@@ -572,33 +459,11 @@ clutter_settings_set_property (GObject      *gobject,
       settings_update_font_name (self);
       break;
 
-    case PROP_FONT_ANTIALIAS:
-      self->xft_antialias = g_value_get_int (value);
-      settings_update_font_options (self);
-      break;
-
     case PROP_FONT_DPI:
       self->font_dpi = g_value_get_int (value);
       settings_update_resolution (self);
       break;
 
-    case PROP_FONT_HINTING:
-      self->xft_hinting = g_value_get_int (value);
-      settings_update_font_options (self);
-      break;
-
-    case PROP_FONT_HINT_STYLE:
-      g_free (self->xft_hint_style);
-      self->xft_hint_style = g_value_dup_string (value);
-      settings_update_font_options (self);
-      break;
-
-    case PROP_FONT_RGBA:
-      g_free (self->xft_rgba);
-      self->xft_rgba = g_value_dup_string (value);
-      settings_update_font_options (self);
-      break;
-
     case PROP_LONG_PRESS_DURATION:
       self->long_press_duration = g_value_get_int (value);
       break;
@@ -607,11 +472,6 @@ clutter_settings_set_property (GObject      *gobject,
       self->password_hint_time = g_value_get_uint (value);
       break;
 
-    case PROP_UNSCALED_FONT_DPI:
-      self->font_dpi = g_value_get_int (value);
-      settings_update_resolution (self);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -644,26 +504,10 @@ clutter_settings_get_property (GObject    *gobject,
       g_value_set_string (value, self->font_name);
       break;
 
-    case PROP_FONT_ANTIALIAS:
-      g_value_set_int (value, self->xft_antialias);
-      break;
-
     case PROP_FONT_DPI:
       g_value_set_int (value, (int) (self->resolution * 1024));
       break;
 
-    case PROP_FONT_HINTING:
-      g_value_set_int (value, self->xft_hinting);
-      break;
-
-    case PROP_FONT_HINT_STYLE:
-      g_value_set_string (value, self->xft_hint_style);
-      break;
-
-    case PROP_FONT_RGBA:
-      g_value_set_string (value, self->xft_rgba);
-      break;
-
     case PROP_LONG_PRESS_DURATION:
       g_value_set_int (value, self->long_press_duration);
       break;
@@ -751,20 +595,6 @@ clutter_settings_class_init (ClutterSettingsClass *klass)
                          G_PARAM_READWRITE |
                          G_PARAM_STATIC_STRINGS);
 
-  /**
-   * ClutterSettings:font-antialias:
-   *
-   * Whether or not to use antialiasing when rendering text; a value
-   * of 1 enables it unconditionally; a value of 0 disables it
-   * unconditionally; and -1 will use the system's default.
-   */
-  obj_props[PROP_FONT_ANTIALIAS] =
-    g_param_spec_int ("font-antialias", NULL, NULL,
-                      -1, 1,
-                      -1,
-                      G_PARAM_READWRITE |
-                      G_PARAM_STATIC_STRINGS);
-
   /**
    * ClutterSettings:font-dpi:
    *
@@ -779,62 +609,6 @@ clutter_settings_class_init (ClutterSettingsClass *klass)
                       G_PARAM_READWRITE |
                       G_PARAM_STATIC_STRINGS);
 
-  obj_props[PROP_UNSCALED_FONT_DPI] =
-    g_param_spec_int ("unscaled-font-dpi", NULL, NULL,
-                      -1, 1024 * 1024,
-                      -1,
-                      G_PARAM_WRITABLE |
-                      G_PARAM_STATIC_STRINGS);
-
-  /**
-   * ClutterSettings:font-hinting:
-   *
-   * Whether or not to use hinting when rendering text; a value of 1
-   * unconditionally enables it; a value of 0 unconditionally disables
-   * it; and a value of -1 will use the system's default.
-   */
-  obj_props[PROP_FONT_HINTING] =
-    g_param_spec_int ("font-hinting", NULL, NULL,
-                      -1, 1,
-                      -1,
-                      G_PARAM_READWRITE |
-                      G_PARAM_STATIC_STRINGS);
-
-  /**
-   * ClutterSettings:font-hint-style:
-   *
-   * The style of the hinting used when rendering text. Valid values
-   * are:
-   *
-   *   - hintnone
-   *   - hintslight
-   *   - hintmedium
-   *   - hintfull
-   */
-  obj_props[PROP_FONT_HINT_STYLE] =
-    g_param_spec_string ("font-hint-style", NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_STATIC_STRINGS);
-
-  /**
-   * ClutterSettings:font-subpixel-order:
-   *
-   * The type of sub-pixel antialiasing used when rendering text. Valid
-   * values are:
-   *
-   *   - none
-   *   - rgb
-   *   - bgr
-   *   - vrgb
-   *   - vbgr
-   */
-  obj_props[PROP_FONT_RGBA] =
-    g_param_spec_string ("font-subpixel-order", NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_STATIC_STRINGS);
-
   /**
    * ClutterSettings:long-press-duration:
    *
@@ -880,7 +654,6 @@ clutter_settings_init (ClutterSettings *self)
   self->resolution = -1.0;
 
   self->font_dpi = -1;
-  self->unscaled_font_dpi = -1;
 
   self->double_click_time = 250;
   self->double_click_distance = 5;
@@ -889,11 +662,6 @@ clutter_settings_init (ClutterSettings *self)
 
   self->font_name = g_strdup (DEFAULT_FONT_NAME);
 
-  self->xft_antialias = -1;
-  self->xft_hinting = -1;
-  self->xft_hint_style = NULL;
-  self->xft_rgba = NULL;
-
   self->long_press_duration = 500;
 }
 
