@@ -33,7 +33,7 @@
 #include "cogl/cogl-frame-info-private.h"
 #include "cogl/cogl-context-private.h"
 
-G_DEFINE_TYPE (CoglFrameInfo, cogl_frame_info, G_TYPE_OBJECT);
+G_DEFINE_FINAL_TYPE (CoglFrameInfo, cogl_frame_info, G_TYPE_OBJECT);
 
 static void
 cogl_frame_info_dispose (GObject *object)
@@ -61,13 +61,15 @@ cogl_frame_info_class_init (CoglFrameInfoClass *class)
 
 CoglFrameInfo *
 cogl_frame_info_new (CoglContext *context,
-                     int64_t      global_frame_counter)
+                     int64_t      global_frame_counter,
+                     int64_t      view_frame_counter)
 {
   CoglFrameInfo *info;
 
   info = g_object_new (COGL_TYPE_FRAME_INFO, NULL);
   info->context = context;
   info->global_frame_counter = global_frame_counter;
+  info->view_frame_counter = view_frame_counter;
 
   return info;
 }
@@ -86,6 +88,17 @@ cogl_frame_info_get_presentation_time_us (CoglFrameInfo *info)
   return info->presentation_time_us;
 }
 
+int64_t
+cogl_frame_info_get_target_presentation_time_us (CoglFrameInfo *info)
+{
+  g_warn_if_fail (!(info->flags & COGL_FRAME_INFO_FLAG_SYMBOLIC));
+
+  if (!info->has_target_presentation_time)
+    return 0;
+
+  return info->target_presentation_time_us;
+}
+
 float
 cogl_frame_info_get_refresh_rate (CoglFrameInfo *info)
 {
@@ -100,6 +113,12 @@ cogl_frame_info_get_global_frame_counter (CoglFrameInfo *info)
   return info->global_frame_counter;
 }
 
+int64_t
+cogl_frame_info_get_view_frame_counter (CoglFrameInfo *info)
+{
+  return info->view_frame_counter;
+}
+
 gboolean
 cogl_frame_info_get_is_symbolic (CoglFrameInfo *info)
 {
