@@ -41,16 +41,18 @@
 #include "cogl/cogl-texture-private.h"
 #include "cogl/cogl-blend-string.h"
 #include "cogl/cogl-journal-private.h"
-#include "cogl/cogl-color-private.h"
+#include "cogl/cogl-color.h"
 #include "cogl/cogl-util.h"
 #include "cogl/cogl-profile.h"
 #include "cogl/cogl-depth-state-private.h"
-#include "cogl/cogl1-context.h"
+#include "cogl/cogl-snippet-private.h"
 
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <string.h>
 
+typedef gboolean (*CoglPipelineChildCallback) (CoglPipeline *child, void *user_data);
+
 static void recursively_free_layer_caches (CoglPipeline *pipeline);
 static gboolean _cogl_pipeline_is_weak (CoglPipeline *pipeline);
 
@@ -62,43 +64,83 @@ const CoglPipelineProgend *_cogl_pipeline_progend;
 #include "cogl/driver/gl/cogl-pipeline-vertend-glsl-private.h"
 #include "cogl/driver/gl/cogl-pipeline-progend-glsl-private.h"
 
-G_DEFINE_FINAL_TYPE (CoglPipeline, cogl_pipeline, COGL_TYPE_NODE)
+G_DEFINE_FINAL_TYPE (CoglPipeline, cogl_pipeline, G_TYPE_OBJECT)
+
+static void
+cogl_pipeline_unparent (CoglPipeline *pipeline)
+{
+  CoglPipeline *old_parent = pipeline->parent;
+
+  if (old_parent)
+    {
+      if (old_parent->first_child == pipeline)
+        old_parent->first_child = pipeline->next_sibling;
+
+      if (old_parent->last_child == pipeline)
+        old_parent->last_child = pipeline->prev_sibling;
+
+      if (pipeline->prev_sibling)
+        pipeline->prev_sibling->next_sibling = pipeline->next_sibling;
+      if (pipeline->next_sibling)
+        pipeline->next_sibling->prev_sibling = pipeline->prev_sibling;
+    }
+
+  if (pipeline->has_parent_reference)
+    g_object_unref (old_parent);
+
+  pipeline->parent = NULL;
+  pipeline->prev_sibling = NULL;
+  pipeline->next_sibling = NULL;
+}
 
 static void
 _cogl_pipeline_revert_weak_ancestors (CoglPipeline *strong)
 {
-  CoglNode *n;
+  CoglPipeline *p;
 
   g_return_if_fail (!strong->is_weak);
 
   /* This reverts the effect of calling
      _cogl_pipeline_promote_weak_ancestors */
 
-  if (COGL_NODE (strong)->parent == NULL)
+  if (strong->parent == NULL)
     return;
 
-  for (n = COGL_NODE (strong)->parent;
+  for (p = strong->parent;
        /* We can assume that all weak pipelines have a parent */
-       COGL_PIPELINE (n)->is_weak;
-       n = n->parent)
-    /* 'n' is weak so we unref its parent */
-    g_object_unref (n->parent);
+       p->is_weak;
+       p = p->parent)
+    /* 'p' is weak so we unref its parent */
+    g_object_unref (p->parent);
+}
+
+static void
+cogl_pipeline_foreach_child (CoglPipeline              *pipeline,
+                             CoglPipelineChildCallback  callback,
+                             void                      *user_data)
+{
+
+  for (CoglPipeline *child = pipeline->first_child;
+       child != NULL;
+       child = child->next_sibling)
+    {
+      if (!callback (child, user_data))
+        break;
+    }
 }
 
 static gboolean
-destroy_weak_children_cb (CoglNode *node,
-                          void *user_data)
+destroy_weak_children_cb (CoglPipeline *pipeline,
+                          void         *user_data)
 {
-  CoglPipeline *pipeline = COGL_PIPELINE (node);
-
   if (_cogl_pipeline_is_weak (pipeline))
     {
-      _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                         destroy_weak_children_cb,
-                                         NULL);
+      cogl_pipeline_foreach_child (pipeline,
+                                   destroy_weak_children_cb,
+                                   NULL);
 
       pipeline->destroy_callback (pipeline, pipeline->destroy_data);
-      _cogl_pipeline_node_unparent_real (COGL_NODE (pipeline));
+      cogl_pipeline_unparent (pipeline);
     }
 
   return TRUE;
@@ -113,13 +155,13 @@ cogl_pipeline_dispose (GObject *object)
     _cogl_pipeline_revert_weak_ancestors (pipeline);
 
   /* Weak pipelines don't take a reference on their parent */
-  _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                     destroy_weak_children_cb,
-                                     NULL);
+  cogl_pipeline_foreach_child (pipeline,
+                               destroy_weak_children_cb,
+                               NULL);
 
-  g_assert (_cogl_list_empty (&COGL_NODE (pipeline)->children));
+  g_assert (pipeline->first_child == NULL);
 
-  _cogl_pipeline_node_unparent_real (COGL_NODE (pipeline));
+  cogl_pipeline_unparent (pipeline);
 
   if (pipeline->differences & COGL_PIPELINE_STATE_USER_SHADER &&
       pipeline->big_state->user_program)
@@ -154,6 +196,8 @@ cogl_pipeline_dispose (GObject *object)
 
   recursively_free_layer_caches (pipeline);
 
+  g_clear_pointer (&pipeline->capabilities, g_array_unref);
+
   G_OBJECT_CLASS (cogl_pipeline_parent_class)->dispose (object);
 }
 
@@ -246,10 +290,10 @@ _cogl_pipeline_init_default_pipeline (CoglContext *context)
 
 
 static gboolean
-recursively_free_layer_caches_cb (CoglNode *node,
-                                  void *user_data)
+recursively_free_layer_caches_cb (CoglPipeline *pipeline,
+                                  gpointer      user_data)
 {
-  recursively_free_layer_caches (COGL_PIPELINE (node));
+  recursively_free_layer_caches (pipeline);
   return TRUE;
 }
 
@@ -271,9 +315,9 @@ recursively_free_layer_caches (CoglPipeline *pipeline)
     g_free (pipeline->layers_cache);
   pipeline->layers_cache_dirty = TRUE;
 
-  _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                     recursively_free_layer_caches_cb,
-                                     NULL);
+  cogl_pipeline_foreach_child (pipeline,
+                               recursively_free_layer_caches_cb,
+                               NULL);
 }
 
 static void
@@ -281,10 +325,34 @@ _cogl_pipeline_set_parent (CoglPipeline *pipeline,
                            CoglPipeline *parent,
                            gboolean take_strong_reference)
 {
-  /* Chain up */
-  _cogl_pipeline_node_set_parent_real (COGL_NODE (pipeline),
-                                       COGL_NODE (parent),
-                                       take_strong_reference);
+  g_autoptr (CoglPipeline) owned_parent = NULL;
+
+  g_assert (COGL_IS_PIPELINE (pipeline));
+  g_assert (COGL_IS_PIPELINE (parent));
+
+  if (pipeline->parent == parent &&
+      pipeline->has_parent_reference == take_strong_reference)
+    return;
+
+  if (pipeline->parent)
+    {
+      owned_parent = g_object_ref (pipeline->parent);
+      cogl_pipeline_unparent (pipeline);
+    }
+
+  pipeline->parent = take_strong_reference ? g_object_ref (parent) : parent;
+  pipeline->has_parent_reference = take_strong_reference;
+
+  if (parent->first_child)
+    {
+      parent->first_child->prev_sibling = pipeline;
+      pipeline->next_sibling = parent->first_child;
+    }
+  else
+    {
+      parent->last_child = pipeline;
+    }
+  parent->first_child = pipeline;
 
   /* Since we just changed the ancestry of the pipeline its cache of
    * layers could now be invalid so free it... */
@@ -295,7 +363,7 @@ _cogl_pipeline_set_parent (CoglPipeline *pipeline,
 static void
 _cogl_pipeline_promote_weak_ancestors (CoglPipeline *strong)
 {
-  CoglNode *n;
+  CoglPipeline *p;
 
   g_return_if_fail (!strong->is_weak);
 
@@ -303,15 +371,15 @@ _cogl_pipeline_promote_weak_ancestors (CoglPipeline *strong)
      taking a reference on strong's grandparent. We don't need to take
      a reference on strong's direct parent */
 
-  if (COGL_NODE (strong)->parent == NULL)
+  if (strong->parent == NULL)
     return;
 
-  for (n = COGL_NODE (strong)->parent;
+  for (p = strong->parent;
        /* We can assume that all weak pipelines have a parent */
-       COGL_PIPELINE (n)->is_weak;
-       n = n->parent)
+       p->is_weak;
+       p = p->parent)
     /* 'n' is weak so we take a reference on its parent */
-    g_object_ref (n->parent);
+    g_object_ref (p->parent);
 }
 
 /* XXX: Always have an eye out for opportunities to lower the cost of
@@ -329,6 +397,11 @@ _cogl_pipeline_copy (CoglPipeline *src, gboolean is_weak)
   pipeline->dirty_real_blend_enable = src->dirty_real_blend_enable;
   pipeline->unknown_color_alpha = src->unknown_color_alpha;
 
+  if (src->capabilities)
+    pipeline->capabilities = g_array_copy (src->capabilities);
+
+  pipeline->name = src->name;
+
   /* XXX:
    * consider generalizing the idea of "cached" properties. These
    * would still have an authority like other sparse properties but
@@ -994,9 +1067,9 @@ _cogl_pipeline_init_multi_property_sparse_state (CoglPipeline *pipeline,
 }
 
 static gboolean
-check_if_strong_cb (CoglNode *node, void *user_data)
+check_if_strong_cb (CoglPipeline *pipeline,
+                    void         *user_data)
 {
-  CoglPipeline *pipeline = COGL_PIPELINE (node);
   gboolean *has_strong_child = user_data;
 
   if (!_cogl_pipeline_is_weak (pipeline))
@@ -1012,9 +1085,9 @@ static gboolean
 has_strong_children (CoglPipeline *pipeline)
 {
   gboolean has_strong_child = FALSE;
-  _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                     check_if_strong_cb,
-                                     &has_strong_child);
+  cogl_pipeline_foreach_child (pipeline,
+                               check_if_strong_cb,
+                               &has_strong_child);
   return has_strong_child;
 }
 
@@ -1028,10 +1101,9 @@ _cogl_pipeline_is_weak (CoglPipeline *pipeline)
 }
 
 static gboolean
-reparent_children_cb (CoglNode *node,
-                      void *user_data)
+reparent_children_cb (CoglPipeline *pipeline,
+                      void         *user_data)
 {
-  CoglPipeline *pipeline = COGL_PIPELINE (node);
   CoglPipeline *parent = user_data;
 
   _cogl_pipeline_set_parent (pipeline, parent, TRUE);
@@ -1076,7 +1148,7 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
           /* XXX: note we use cogl_flush() not _cogl_flush_journal() so
            * we will flush *all* known journals that might reference the
            * current pipeline. */
-          cogl_flush ();
+          cogl_context_flush (pipeline->context);
         }
     }
 
@@ -1139,14 +1211,14 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
   /* The simplest descendants to handle are weak pipelines; we simply
    * destroy them if we are modifying a pipeline they depend on. This
    * means weak pipelines never cause us to do a copy-on-write. */
-  _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                     destroy_weak_children_cb,
-                                     NULL);
+  cogl_pipeline_foreach_child (pipeline,
+                               destroy_weak_children_cb,
+                               NULL);
 
   /* If there are still children remaining though we'll need to
    * perform a copy-on-write and reparent the dependants as children
    * of the copy. */
-  if (!_cogl_list_empty (&COGL_NODE (pipeline)->children))
+  if (pipeline->first_child != NULL)
     {
       CoglPipeline *new_authority;
 
@@ -1182,9 +1254,9 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
 
       /* Reparent the dependants of pipeline to be children of
        * new_authority instead... */
-      _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
-                                         reparent_children_cb,
-                                         new_authority);
+      cogl_pipeline_foreach_child (pipeline,
+                                   reparent_children_cb,
+                                   new_authority);
 
       /* The children will keep the new authority alive so drop the
        * reference we got when copying... */
@@ -2350,11 +2422,12 @@ _cogl_pipeline_init_layer_state_hash_functions (void)
     _cogl_pipeline_layer_hash_fragment_snippets_state;
 
   {
-  /* So we get a big error if we forget to update this code! */
-  _COGL_STATIC_ASSERT (COGL_PIPELINE_LAYER_STATE_SPARSE_COUNT == 9,
-                       "Don't forget to install a hash function for new "
-                       "pipeline state and update assert at end of "
-                       "_cogl_pipeline_init_state_hash_functions");
+    /* So we get a big error if we forget to update this code!
+     * Make sure to install a hash function for newly added
+     * pipeline state and update assert
+     * in _cogl_pipeline_init_state_hash_functions
+     */
+    G_STATIC_ASSERT (COGL_PIPELINE_STATE_SPARSE_COUNT == 14);
   }
 }
 
@@ -2458,11 +2531,12 @@ _cogl_pipeline_init_state_hash_functions (void)
     _cogl_pipeline_hash_fragment_snippets_state;
 
   {
-  /* So we get a big error if we forget to update this code! */
-  _COGL_STATIC_ASSERT (COGL_PIPELINE_STATE_SPARSE_COUNT == 14,
-                       "Make sure to install a hash function for "
-                       "newly added pipeline state and update assert "
-                       "in _cogl_pipeline_init_state_hash_functions");
+    /* So we get a big error if we forget to update this code!
+     * Make sure to install a hash function for newly added
+     * pipeline state and update assert
+     * in _cogl_pipeline_init_state_hash_functions
+     */
+    G_STATIC_ASSERT (COGL_PIPELINE_STATE_SPARSE_COUNT == 14);
   }
 }
 
@@ -2549,7 +2623,7 @@ deep_copy_layer_cb (CoglPipelineLayer *src_layer,
           differences ^= to_copy;
         }
 
-      src_layer = COGL_PIPELINE_LAYER (COGL_NODE (src_layer)->parent);
+      src_layer = src_layer->parent;
     }
 
   return TRUE;
@@ -2576,7 +2650,7 @@ _cogl_pipeline_deep_copy (CoglPipeline *pipeline,
 
   for (authority = pipeline;
        authority != ctx->default_pipeline && differences;
-       authority = COGL_PIPELINE (COGL_NODE (authority)->parent))
+       authority = authority->parent)
     {
       unsigned long to_copy = differences & authority->differences;
 
@@ -2792,3 +2866,73 @@ cogl_pipeline_get_uniform_location (CoglPipeline *pipeline,
 
   return ctx->n_uniform_names++;
 }
+
+typedef struct
+{
+  GQuark domain;
+  unsigned int capability;
+} CapabilityEntry;
+
+static void
+cogl_pipeline_add_capability (CoglPipeline *pipeline,
+                              GQuark        domain,
+                              unsigned int  capability)
+{
+  CapabilityEntry entry = {
+    .domain = domain,
+    .capability = capability,
+  };
+
+  if (!pipeline->capabilities)
+    pipeline->capabilities = g_array_new (FALSE, FALSE, sizeof (entry));
+
+  g_array_append_val (pipeline->capabilities, entry);
+}
+
+void
+cogl_pipeline_add_capability_from_snippet (CoglPipeline *pipeline,
+                                           CoglSnippet  *snippet)
+{
+  GQuark capability_domain;
+  unsigned int capability;
+
+  if (cogl_snippet_get_capability (snippet, &capability_domain, &capability))
+    cogl_pipeline_add_capability (pipeline, capability_domain, capability);
+}
+
+gboolean
+cogl_pipeline_has_capability (CoglPipeline *pipeline,
+                              GQuark        domain,
+                              unsigned int  capability)
+{
+  int i;
+
+  if (!pipeline->capabilities)
+    return FALSE;
+
+  for (i = 0; i < pipeline->capabilities->len; i++)
+    {
+      CapabilityEntry *entry = &g_array_index (pipeline->capabilities,
+                                               CapabilityEntry,
+                                               i);
+
+      if (entry->domain == domain &&
+          entry->capability == capability)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
+void
+cogl_pipeline_set_static_name (CoglPipeline *pipeline,
+                               const char   *name)
+{
+  pipeline->name = name;
+}
+
+const char *
+cogl_pipeline_get_name (CoglPipeline *pipeline)
+{
+  return pipeline->name;
+}
