@@ -89,6 +89,8 @@ cogl_onscreen_egl_dispose (GObject *object)
   CoglRenderer *renderer = context->display->renderer;
   CoglRendererEGL *egl_renderer = renderer->winsys;
 
+  G_OBJECT_CLASS (cogl_onscreen_egl_parent_class)->dispose (object);
+
   if (priv->egl_surface != EGL_NO_SURFACE)
     {
       /* Cogl always needs a valid context bound to something so if we
@@ -111,8 +113,6 @@ cogl_onscreen_egl_dispose (GObject *object)
         g_warning ("Failed to destroy EGL surface");
       priv->egl_surface = EGL_NO_SURFACE;
     }
-
-  G_OBJECT_CLASS (cogl_onscreen_egl_parent_class)->dispose (object);
 }
 
 static void
@@ -201,11 +201,10 @@ cogl_onscreen_egl_get_buffer_age (CoglOnscreen *onscreen)
 }
 
 static void
-cogl_onscreen_egl_swap_region (CoglOnscreen  *onscreen,
-                               const int     *user_rectangles,
-                               int            n_rectangles,
-                               CoglFrameInfo *info,
-                               gpointer       user_data)
+cogl_onscreen_egl_swap_region (CoglOnscreen    *onscreen,
+                               const MtkRegion *region,
+                               CoglFrameInfo   *info,
+                               gpointer         user_data)
 {
   CoglOnscreenEgl *onscreen_egl = COGL_ONSCREEN_EGL (onscreen);
   CoglOnscreenEglPrivate *priv =
@@ -214,19 +213,14 @@ cogl_onscreen_egl_swap_region (CoglOnscreen  *onscreen,
   CoglContext *context = cogl_framebuffer_get_context (framebuffer);
   CoglRenderer *renderer = context->display->renderer;
   CoglRendererEGL *egl_renderer = renderer->winsys;
-  int framebuffer_height  = cogl_framebuffer_get_height (framebuffer);
-  int *rectangles = g_alloca (sizeof (int) * n_rectangles * 4);
-  int i;
+  int n_rectangles;
+  int *egl_rectangles;
 
-  /* eglSwapBuffersRegion expects rectangles relative to the
-   * bottom left corner but we are given rectangles relative to
-   * the top left so we need to flip them... */
-  memcpy (rectangles, user_rectangles, sizeof (int) * n_rectangles * 4);
-  for (i = 0; i < n_rectangles; i++)
-    {
-      int *rect = &rectangles[4 * i];
-      rect[1] = framebuffer_height - rect[1] - rect[3];
-    }
+  n_rectangles = mtk_region_num_rectangles (region);
+  egl_rectangles = g_alloca (n_rectangles * sizeof (int) * 4);
+  cogl_region_to_flipped_array (region,
+                                cogl_framebuffer_get_height (framebuffer),
+                                egl_rectangles);
 
   /* At least for eglSwapBuffers the EGL spec says that the surface to
      swap must be bound to the current context. It looks like Mesa
@@ -240,7 +234,7 @@ cogl_onscreen_egl_swap_region (CoglOnscreen  *onscreen,
   if (egl_renderer->pf_eglSwapBuffersRegion (egl_renderer->edpy,
                                              priv->egl_surface,
                                              n_rectangles,
-                                             rectangles) == EGL_FALSE)
+                                             egl_rectangles) == EGL_FALSE)
     g_warning ("Error reported by eglSwapBuffersRegion");
 
   /* Update latest sync object after buffer swap */
@@ -248,9 +242,8 @@ cogl_onscreen_egl_swap_region (CoglOnscreen  *onscreen,
 }
 
 static void
-cogl_onscreen_egl_queue_damage_region (CoglOnscreen *onscreen,
-                                       const int    *rectangles,
-                                       int           n_rectangles)
+cogl_onscreen_egl_queue_damage_region (CoglOnscreen    *onscreen,
+                                       const MtkRegion *region)
 {
   CoglOnscreenEgl *onscreen_egl = COGL_ONSCREEN_EGL (onscreen);
   CoglOnscreenEglPrivate *priv =
@@ -259,15 +252,25 @@ cogl_onscreen_egl_queue_damage_region (CoglOnscreen *onscreen,
   CoglContext *context = cogl_framebuffer_get_context (framebuffer);
   CoglRenderer *renderer = context->display->renderer;
   CoglRendererEGL *egl_renderer = renderer->winsys;
-
-  g_return_if_fail (n_rectangles > 0);
+  int n_rectangles;
+  int *egl_rectangles;
 
   if (!egl_renderer->pf_eglSetDamageRegion)
     return;
 
+  g_return_if_fail (region);
+
+  n_rectangles = mtk_region_num_rectangles (region);
+  g_return_if_fail (n_rectangles > 0);
+
+  egl_rectangles = g_alloca (n_rectangles * sizeof (int) * 4);
+  cogl_region_to_flipped_array (region,
+                                cogl_framebuffer_get_height (framebuffer),
+                                egl_rectangles);
+
   if (egl_renderer->pf_eglSetDamageRegion (egl_renderer->edpy,
                                            priv->egl_surface,
-                                           rectangles,
+                                           egl_rectangles,
                                            n_rectangles) == EGL_FALSE)
     g_warning ("Error reported by eglSetDamageRegion");
 }
@@ -294,11 +297,10 @@ cogl_onscreen_egl_maybe_create_timestamp_query (CoglOnscreen  *onscreen,
 }
 
 static void
-cogl_onscreen_egl_swap_buffers_with_damage (CoglOnscreen  *onscreen,
-                                            const int     *rectangles,
-                                            int            n_rectangles,
-                                            CoglFrameInfo *info,
-                                            gpointer       user_data)
+cogl_onscreen_egl_swap_buffers_with_damage (CoglOnscreen    *onscreen,
+                                            const MtkRegion *region,
+                                            CoglFrameInfo   *info,
+                                            gpointer         user_data)
 {
   CoglOnscreenEgl *onscreen_egl = COGL_ONSCREEN_EGL (onscreen);
   CoglOnscreenEglPrivate *priv =
@@ -321,25 +323,20 @@ cogl_onscreen_egl_swap_buffers_with_damage (CoglOnscreen  *onscreen,
                                         framebuffer,
                                         COGL_FRAMEBUFFER_STATE_BIND);
 
-  if (n_rectangles && priv->pf_eglSwapBuffersWithDamage)
+  if (region && priv->pf_eglSwapBuffersWithDamage)
     {
-      size_t size = n_rectangles * sizeof (int) * 4;
-      int *flipped = alloca (size);
-      int i;
+      int n_rectangles;
+      int *egl_rectangles;
 
-      memcpy (flipped, rectangles, size);
-      for (i = 0; i < n_rectangles; i++)
-        {
-          const int *rect = rectangles + 4 * i;
-          int *flip_rect = flipped + 4 * i;
-
-          flip_rect[1] =
-            cogl_framebuffer_get_height (framebuffer) - rect[1] - rect[3];
-        }
+      n_rectangles = mtk_region_num_rectangles (region);
+      egl_rectangles = alloca (n_rectangles * sizeof (int) * 4);
+      cogl_region_to_flipped_array (region,
+                                    cogl_framebuffer_get_height (framebuffer),
+                                    egl_rectangles);
 
       if (priv->pf_eglSwapBuffersWithDamage (egl_renderer->edpy,
                                              priv->egl_surface,
-                                             flipped,
+                                             egl_rectangles,
                                              n_rectangles) == EGL_FALSE)
         g_warning ("Error reported by eglSwapBuffersWithDamage");
     }
