@@ -31,8 +31,6 @@ G_DECLARE_FINAL_TYPE (MetaColorDevice, meta_color_device,
 MetaColorDevice * meta_color_device_new (MetaColorManager *color_manager,
                                          MetaMonitor      *monitor);
 
-void meta_color_device_destroy (MetaColorDevice *color_device);
-
 void meta_color_device_update_monitor (MetaColorDevice *color_device,
                                        MetaMonitor     *monitor);
 
@@ -42,6 +40,8 @@ const char * meta_color_device_get_id (MetaColorDevice *color_device);
 META_EXPORT_TEST
 MetaMonitor * meta_color_device_get_monitor (MetaColorDevice *color_device);
 
+ClutterColorState * meta_color_device_get_color_state (MetaColorDevice *color_device);
+
 META_EXPORT_TEST
 MetaColorProfile * meta_color_device_get_device_profile (MetaColorDevice *color_device);
 
@@ -61,8 +61,7 @@ MetaColorProfile * meta_color_device_generate_profile_finish (MetaColorDevice  *
 META_EXPORT_TEST
 gboolean meta_color_device_is_ready (MetaColorDevice *color_device);
 
-void meta_color_device_update (MetaColorDevice *color_device,
-                               unsigned int     temperature);
+void meta_color_device_update (MetaColorDevice *color_device);
 
 META_EXPORT_TEST
 void meta_set_color_efivar_test_path (const char *path);
