@@ -306,7 +306,7 @@ meta_color_profile_new_from_icc (MetaColorManager     *color_manager,
   return color_profile;
 }
 
-static gboolean
+static void
 notify_ready_idle (gpointer user_data)
 {
   MetaColorProfile *color_profile = user_data;
@@ -314,8 +314,6 @@ notify_ready_idle (gpointer user_data)
   color_profile->notify_ready_id = 0;
   color_profile->is_ready = TRUE;
   g_signal_emit (color_profile, signals[READY], 0, TRUE);
-
-  return G_SOURCE_REMOVE;
 }
 
 MetaColorProfile *
@@ -341,8 +339,8 @@ meta_color_profile_new_from_cd_profile (MetaColorManager     *color_manager,
   color_profile->cd_profile_id = g_strdup_printf ("icc-%s", checksum);
   color_profile->cd_profile = g_object_ref (cd_profile);
 
-  color_profile->notify_ready_id = g_idle_add (notify_ready_idle,
-                                               color_profile);
+  color_profile->notify_ready_id = g_idle_add_once (notify_ready_idle,
+                                                    color_profile);
 
   return color_profile;
 }
