@@ -14,14 +14,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_DBUS_SESSION_WATCHER_H
-#define META_DBUS_SESSION_WATCHER_H
+#pragma once
 
 #include <glib-object.h>
 
@@ -68,5 +65,3 @@ MetaDbusSessionManager * meta_dbus_session_manager (MetaDbusSessionManager *sess
 char * meta_dbus_session_get_peer_name (MetaDbusSession *session);
 
 char * meta_dbus_session_get_id (MetaDbusSession *session);
-
-#endif /* META_DBUS_SESSION_WATCHER_H */
