@@ -92,15 +92,26 @@ meta_renderer_create_cogl_renderer (MetaRenderer *renderer)
 }
 
 static MetaRendererView *
-meta_renderer_create_view (MetaRenderer       *renderer,
-                           MetaLogicalMonitor *logical_monitor,
-                           MetaOutput         *output,
-                           MetaCrtc           *crtc)
+meta_renderer_create_view (MetaRenderer        *renderer,
+                           MetaLogicalMonitor  *logical_monitor,
+                           MetaMonitor         *monitor,
+                           MetaOutput          *output,
+                           MetaCrtc            *crtc,
+                           GError             **error)
 {
-  return META_RENDERER_GET_CLASS (renderer)->create_view (renderer,
+  MetaRendererView *view;
+
+  view = META_RENDERER_GET_CLASS (renderer)->create_view (renderer,
                                                           logical_monitor,
+                                                          monitor,
                                                           output,
-                                                          crtc);
+                                                          crtc,
+                                                          error);
+
+  if (view)
+    meta_renderer_add_view (renderer, view);
+
+  return view;
 }
 
 /**
@@ -127,9 +138,21 @@ create_crtc_view (MetaLogicalMonitor *logical_monitor,
 {
   MetaRenderer *renderer = user_data;
   MetaRendererView *view;
+  g_autoptr (GError) error = NULL;
 
-  view = meta_renderer_create_view (renderer, logical_monitor, output, crtc);
-  meta_renderer_add_view (renderer, view);
+  view = meta_renderer_create_view (renderer,
+                                    logical_monitor,
+                                    monitor,
+                                    output,
+                                    crtc,
+                                    &error);
+  if (!view)
+    {
+      g_warning ("Failed to create view for %s on %s: %s",
+                 meta_monitor_get_display_name (monitor),
+                 meta_output_get_name (output),
+                 error->message);
+    }
 }
 
 static void
