@@ -150,6 +150,15 @@ typedef struct _MetaScreenCastStreamSrcPrivate
   GHashTable *modifiers;
 } MetaScreenCastStreamSrcPrivate;
 
+static void meta_screen_cast_stream_src_init_initable_iface (GInitableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (MetaScreenCastStreamSrc,
+                         meta_screen_cast_stream_src,
+                         G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+                                                meta_screen_cast_stream_src_init_initable_iface)
+                         G_ADD_PRIVATE (MetaScreenCastStreamSrc))
+
 static const struct {
   CoglPixelFormat cogl_format;
   enum spa_video_format spa_video_format;
@@ -281,16 +290,6 @@ push_format_object (enum spa_video_format  format,
   return spa_pod_builder_pop (&pod_builder.b, &pod_frame);
 }
 
-static void
-meta_screen_cast_stream_src_init_initable_iface (GInitableIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (MetaScreenCastStreamSrc,
-                         meta_screen_cast_stream_src,
-                         G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
-                                                meta_screen_cast_stream_src_init_initable_iface)
-                         G_ADD_PRIVATE (MetaScreenCastStreamSrc))
-
 static gboolean
 meta_screen_cast_stream_src_get_specs (MetaScreenCastStreamSrc *src,
                                        int                     *width,
@@ -389,7 +388,7 @@ draw_cursor_sprite_via_offscreen (MetaScreenCastStreamSrc  *src,
 
   bitmap_texture = cogl_texture_2d_new_with_size (cogl_context,
                                                   bitmap_width, bitmap_height);
-  cogl_texture_set_auto_mipmap (bitmap_texture, FALSE);
+  cogl_texture_2d_set_auto_mipmap (COGL_TEXTURE_2D (bitmap_texture), FALSE);
   if (!cogl_texture_allocate (bitmap_texture, error))
     {
       g_object_unref (bitmap_texture);
@@ -693,7 +692,7 @@ meta_screen_cast_stream_src_calculate_stride (MetaScreenCastStreamSrc *src,
       dmabuf_handle = g_hash_table_lookup (priv->dmabuf_handles,
                                            GINT_TO_POINTER (spa_data->fd));
       g_assert (dmabuf_handle != NULL);
-      return cogl_dma_buf_handle_get_stride (dmabuf_handle);
+      return cogl_dma_buf_handle_get_stride (dmabuf_handle, 0);
     }
 
   if (!cogl_pixel_format_from_spa_video_format (priv->video_format.format,
@@ -724,7 +723,7 @@ maybe_set_sync_points (MetaScreenCastStreamSrc *src,
   ClutterBackend *clutter_backend = meta_backend_get_clutter_backend (backend);
   CoglContext *cogl_context =
     clutter_backend_get_cogl_context (clutter_backend);
-  g_autofd int sync_fd = -1;
+  int sync_fd;
   g_autoptr (GError) local_error = NULL;
 
   sync_timeline = spa_buffer_find_meta_data (spa_buffer,
@@ -821,7 +820,7 @@ meta_screen_cast_stream_src_pending_follow_up_frame (MetaScreenCastStreamSrc *sr
   return priv->follow_up_frame_source_id != 0;
 }
 
-static gboolean
+static void
 follow_up_frame_cb (gpointer user_data)
 {
   MetaScreenCastStreamSrc *src = user_data;
@@ -830,8 +829,6 @@ follow_up_frame_cb (gpointer user_data)
 
   priv->follow_up_frame_source_id = 0;
   meta_screen_cast_stream_src_record_follow_up (src);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -844,9 +841,9 @@ maybe_schedule_follow_up_frame (MetaScreenCastStreamSrc *src,
   if (priv->follow_up_frame_source_id)
     return;
 
-  priv->follow_up_frame_source_id = g_timeout_add (us2ms (timeout_us),
-                                                   follow_up_frame_cb,
-                                                   src);
+  priv->follow_up_frame_source_id = g_timeout_add_once (us2ms (timeout_us),
+                                                        follow_up_frame_cb,
+                                                        src);
 }
 
 static void
@@ -1450,46 +1447,6 @@ add_video_damage_meta_param (GPtrArray *params)
   g_ptr_array_add (params, g_steal_pointer (&pod));
 }
 
-static gboolean
-explicit_sync_supported (MetaScreenCastStreamSrc *src)
-{
-  uint64_t supported = 0;
-#ifdef HAVE_NATIVE_BACKEND
-  MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
-  MetaScreenCastSession *session =
-    meta_screen_cast_stream_get_session (stream);
-  MetaScreenCast *screen_cast =
-    meta_screen_cast_session_get_screen_cast (session);
-  MetaBackend *backend = meta_screen_cast_get_backend (screen_cast);
-  ClutterBackend *clutter_backend = meta_backend_get_clutter_backend (backend);
-  CoglContext *cogl_context =
-    clutter_backend_get_cogl_context (clutter_backend);
-  CoglRenderer *cogl_renderer;
-  CoglRendererEGL *cogl_renderer_egl;
-  MetaRendererNativeGpuData *renderer_gpu_data;
-  MetaRenderDevice *render_device;
-  MetaDeviceFile *device_file;
-  int drm_fd;
-
-  if (!cogl_context_has_feature (cogl_context, COGL_FEATURE_ID_SYNC_FD))
-    return FALSE;
-
-  cogl_renderer = cogl_context_get_renderer (cogl_context);
-  cogl_renderer_egl = cogl_renderer->winsys;
-  renderer_gpu_data = cogl_renderer_egl->platform;
-  render_device = renderer_gpu_data->render_device;
-  device_file = meta_render_device_get_device_file (render_device);
-  if (!device_file)
-    return FALSE;
-
-  drm_fd = meta_device_file_get_fd (device_file);
-  if (drmGetCap (drm_fd, DRM_CAP_SYNCOBJ_TIMELINE, &supported) != 0)
-    return FALSE;
-#endif /* HAVE_NATIVE_BACKEND */
-
-  return supported != 0;
-}
-
 static void
 on_stream_param_changed (void                 *data,
                          uint32_t              id,
@@ -1645,26 +1602,14 @@ on_stream_param_changed (void                 *data,
     SPA_PARAM_META_size, SPA_POD_Int (sizeof (struct spa_meta_header)));
   g_ptr_array_add (params, g_steal_pointer (&pod));
 
-  if (explicit_sync_supported (src))
-    {
-      spa_pod_dynamic_builder_init (&pod_builder, NULL, 0, 1024);
-      pod = spa_pod_builder_add_object (
-        &pod_builder.b,
-        SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
-        SPA_PARAM_META_type, SPA_POD_Id (SPA_META_SyncTimeline),
-        SPA_PARAM_META_size, SPA_POD_Int (sizeof (struct spa_meta_sync_timeline)));
-      g_ptr_array_add (params, g_steal_pointer (&pod));
-
-      meta_topic (META_DEBUG_SCREEN_CAST,
-                  "Advertising explicit sync support for pw_stream %u",
-                  pw_stream_get_node_id (priv->pipewire_stream));
-    }
-  else
-    {
-      meta_topic (META_DEBUG_SCREEN_CAST,
-                  "Not advertising explicit sync support for pw_stream %u",
-                  pw_stream_get_node_id (priv->pipewire_stream));
-    }
+  /* we support Explicit sync */
+  spa_pod_dynamic_builder_init (&pod_builder, NULL, 0, 1024);
+  pod = spa_pod_builder_add_object (
+    &pod_builder.b,
+    SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
+    SPA_PARAM_META_type, SPA_POD_Id (SPA_META_SyncTimeline),
+    SPA_PARAM_META_size, SPA_POD_Int (sizeof (struct spa_meta_sync_timeline)));
+  g_ptr_array_add (params, g_steal_pointer (&pod));
 
   add_video_damage_meta_param (params);
 
@@ -1811,7 +1756,7 @@ on_stream_add_buffer (void             *data,
 
       spa_data->type = SPA_DATA_DmaBuf;
       spa_data->flags = SPA_DATA_FLAG_READWRITE;
-      spa_data->fd = cogl_dma_buf_handle_get_fd (dmabuf_handle);
+      spa_data->fd = cogl_dma_buf_handle_get_fd (dmabuf_handle, 0);
 
       g_hash_table_insert (priv->dmabuf_handles,
                            GINT_TO_POINTER (spa_data->fd),
