@@ -40,6 +40,12 @@ struct _MetaScreenCastWindowStreamSrc
   gulong prepare_frame_handler_id;
 
   gboolean cursor_bitmap_invalid;
+
+  struct {
+    gboolean set;
+    int x;
+    int y;
+  } last_cursor_matadata;
 };
 
 G_DEFINE_TYPE (MetaScreenCastWindowStreamSrc,
@@ -127,8 +133,11 @@ maybe_draw_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
   GError *error = NULL;
   cairo_surface_t *stream_surface;
   int width, height;
-  float scale;
-  MetaMonitorTransform transform;
+  int texture_width, texture_height;
+  float scale, view_scale, cursor_scale;
+  MtkMonitorTransform cursor_transform;
+  const graphene_rect_t *src_rect;
+  graphene_matrix_t matrix;
   int hotspot_x, hotspot_y;
   cairo_t *cr;
 
@@ -145,23 +154,63 @@ maybe_draw_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
   if (!meta_screen_cast_window_transform_cursor_position (screen_cast_window,
                                                           cursor_sprite,
                                                           &cursor_position,
-                                                          &scale,
-                                                          &transform,
-                                                          &relative_cursor_position))
+                                                          &relative_cursor_position,
+                                                          &view_scale))
     return;
 
   meta_cursor_sprite_get_hotspot (cursor_sprite, &hotspot_x, &hotspot_y);
+  cursor_scale = meta_cursor_sprite_get_texture_scale (cursor_sprite);
+  scale = cursor_scale * view_scale;
+  cursor_transform = meta_cursor_sprite_get_texture_transform (cursor_sprite);
+  src_rect = meta_cursor_sprite_get_viewport_src_rect (cursor_sprite);
+
+  texture_width = cogl_texture_get_width (cursor_texture);
+  texture_height = cogl_texture_get_height (cursor_texture);
+
+  if (meta_cursor_sprite_get_viewport_dst_size (cursor_sprite,
+                                                &width,
+                                                &height))
+    {
+      width = (int) ceilf (width * view_scale);
+      height = (int) ceilf (height * view_scale);
+    }
+  else if (src_rect)
+    {
+      width = (int) ceilf (src_rect->size.width * view_scale);
+      height = (int) ceilf (src_rect->size.height * view_scale);
+    }
+  else
+    {
+      if (mtk_monitor_transform_is_rotated (cursor_transform))
+        {
+          width = (int) ceilf (texture_height * scale);
+          height = (int) ceilf (texture_width * scale);
+        }
+      else
+        {
+          width = (int) ceilf (texture_width * scale);
+          height = (int) ceilf (texture_height * scale);
+        }
+    }
+
+  graphene_matrix_init_identity (&matrix);
+  mtk_compute_viewport_matrix (&matrix,
+                               texture_width,
+                               texture_height,
+                               cursor_scale,
+                               cursor_transform,
+                               src_rect);
+
 
-  width = cogl_texture_get_width (cursor_texture) * scale;
-  height = cogl_texture_get_height (cursor_texture) * scale;
   cursor_surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
                                                width, height);
 
   cursor_surface_data = cairo_image_surface_get_data (cursor_surface);
   if (!meta_screen_cast_stream_src_draw_cursor_into (src,
                                                      cursor_texture,
-                                                     scale,
-                                                     transform,
+                                                     width,
+                                                     height,
+                                                     &matrix,
                                                      cursor_surface_data,
                                                      &error))
     {
@@ -195,8 +244,10 @@ maybe_blit_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
                           MtkRectangle                  *stream_rect)
 {
   MetaBackend *backend = get_backend (window_src);
+  ClutterBackend *clutter_backend =
+    meta_backend_get_clutter_backend (backend);
   CoglContext *cogl_context =
-    clutter_backend_get_cogl_context (clutter_get_default_backend ());
+    clutter_backend_get_cogl_context (clutter_backend);
   MetaCursorRenderer *cursor_renderer =
     meta_backend_get_cursor_renderer (backend);
   MetaCursorTracker *cursor_tracker =
@@ -208,11 +259,12 @@ maybe_blit_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
   CoglTexture *cursor_texture;
   CoglPipeline *pipeline;
   int width, height;
-  float scale;
-  MetaMonitorTransform transform;
+  float scale, view_scale, cursor_scale;
+  MtkMonitorTransform cursor_transform;
+  const graphene_rect_t *src_rect;
+  graphene_matrix_t matrix;
   int hotspot_x, hotspot_y;
   float x, y;
-  graphene_matrix_t matrix;
 
   cursor_sprite = meta_cursor_renderer_get_cursor (cursor_renderer);
   if (!cursor_sprite)
@@ -227,12 +279,15 @@ maybe_blit_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
   if (!meta_screen_cast_window_transform_cursor_position (screen_cast_window,
                                                           cursor_sprite,
                                                           &cursor_position,
-                                                          &scale,
-                                                          &transform,
-                                                          &relative_cursor_position))
+                                                          &relative_cursor_position,
+                                                          &view_scale))
     return;
 
   meta_cursor_sprite_get_hotspot (cursor_sprite, &hotspot_x, &hotspot_y);
+  cursor_scale = meta_cursor_sprite_get_texture_scale (cursor_sprite);
+  scale = cursor_scale * view_scale;
+  cursor_transform = meta_cursor_sprite_get_texture_transform (cursor_sprite);
+  src_rect = meta_cursor_sprite_get_viewport_src_rect (cursor_sprite);
 
   x = (relative_cursor_position.x - hotspot_x) * scale;
   y = (relative_cursor_position.y - hotspot_y) * scale;
@@ -246,8 +301,12 @@ maybe_blit_cursor_sprite (MetaScreenCastWindowStreamSrc *window_src,
                                    COGL_PIPELINE_FILTER_LINEAR);
 
   graphene_matrix_init_identity (&matrix);
-  meta_monitor_transform_transform_matrix (transform,
-                                           &matrix);
+  mtk_compute_viewport_matrix (&matrix,
+                               width,
+                               height,
+                               cursor_scale,
+                               cursor_transform,
+                               src_rect);
   cogl_pipeline_set_layer_matrix (pipeline, 0, &matrix);
 
   cogl_framebuffer_draw_rectangle (framebuffer,
@@ -570,6 +629,51 @@ meta_screen_cast_window_stream_record_follow_up (MetaScreenCastStreamSrc *src)
                                                   NULL);
 }
 
+static gboolean
+meta_screen_cast_window_stream_src_is_cursor_metadata_valid (MetaScreenCastStreamSrc *src)
+{
+  MetaScreenCastWindowStreamSrc *window_src =
+    META_SCREEN_CAST_WINDOW_STREAM_SRC (src);
+  MetaScreenCastWindow *screen_cast_window = window_src->screen_cast_window;
+  MetaBackend *backend = get_backend (window_src);
+  MetaCursorRenderer *cursor_renderer =
+    meta_backend_get_cursor_renderer (backend);
+  MetaCursorTracker *cursor_tracker =
+    meta_backend_get_cursor_tracker (backend);
+  MetaCursorSprite *cursor_sprite;
+  graphene_point_t cursor_position;
+  graphene_point_t relative_cursor_position;
+
+  cursor_sprite = meta_cursor_renderer_get_cursor (cursor_renderer);
+  meta_cursor_tracker_get_pointer (cursor_tracker, &cursor_position, NULL);
+
+  if (meta_cursor_tracker_get_pointer_visible (cursor_tracker) &&
+      meta_screen_cast_window_transform_cursor_position (screen_cast_window,
+                                                         cursor_sprite,
+                                                         &cursor_position,
+                                                         &relative_cursor_position,
+                                                         NULL))
+    {
+      int x, y;
+
+      if (!window_src->last_cursor_matadata.set)
+        return FALSE;
+
+      if (window_src->cursor_bitmap_invalid)
+        return FALSE;
+
+      x = (int) roundf (relative_cursor_position.x);
+      y = (int) roundf (relative_cursor_position.y);
+
+      return (window_src->last_cursor_matadata.x == x &&
+              window_src->last_cursor_matadata.y == y);
+    }
+  else
+    {
+      return !window_src->last_cursor_matadata.set;
+    }
+}
+
 static void
 meta_screen_cast_window_stream_src_set_cursor_metadata (MetaScreenCastStreamSrc *src,
                                                         struct spa_meta_cursor  *spa_meta_cursor)
@@ -584,8 +688,7 @@ meta_screen_cast_window_stream_src_set_cursor_metadata (MetaScreenCastStreamSrc
   MetaScreenCastWindow *screen_cast_window = window_src->screen_cast_window;
   MetaCursorSprite *cursor_sprite;
   graphene_point_t cursor_position;
-  float scale;
-  MetaMonitorTransform transform;
+  float view_scale;
   graphene_point_t relative_cursor_position;
   int x, y;
 
@@ -596,10 +699,10 @@ meta_screen_cast_window_stream_src_set_cursor_metadata (MetaScreenCastStreamSrc
       !meta_screen_cast_window_transform_cursor_position (screen_cast_window,
                                                           cursor_sprite,
                                                           &cursor_position,
-                                                          &scale,
-                                                          &transform,
-                                                          &relative_cursor_position))
+                                                          &relative_cursor_position,
+                                                          &view_scale))
     {
+      window_src->last_cursor_matadata.set = FALSE;
       meta_screen_cast_stream_src_unset_cursor_metadata (src,
                                                          spa_meta_cursor);
       return;
@@ -608,6 +711,10 @@ meta_screen_cast_window_stream_src_set_cursor_metadata (MetaScreenCastStreamSrc
   x = (int) roundf (relative_cursor_position.x);
   y = (int) roundf (relative_cursor_position.y);
 
+  window_src->last_cursor_matadata.set = TRUE;
+  window_src->last_cursor_matadata.x = x;
+  window_src->last_cursor_matadata.y = y;
+
   if (window_src->cursor_bitmap_invalid)
     {
       if (cursor_sprite)
@@ -616,8 +723,7 @@ meta_screen_cast_window_stream_src_set_cursor_metadata (MetaScreenCastStreamSrc
                                                                   spa_meta_cursor,
                                                                   cursor_sprite,
                                                                   x, y,
-                                                                  scale,
-                                                                  transform);
+                                                                  view_scale);
         }
       else
         {
@@ -672,6 +778,8 @@ meta_screen_cast_window_stream_src_class_init (MetaScreenCastWindowStreamSrcClas
   src_class->record_follow_up =
     meta_screen_cast_window_stream_record_follow_up;
   src_class->get_videocrop = meta_screen_cast_window_stream_src_get_videocrop;
+  src_class->is_cursor_metadata_valid =
+    meta_screen_cast_window_stream_src_is_cursor_metadata_valid;
   src_class->set_cursor_metadata = meta_screen_cast_window_stream_src_set_cursor_metadata;
   src_class->get_preferred_format =
     meta_screen_cast_window_stream_src_get_preferred_format;
