@@ -20,6 +20,7 @@
 #include "backends/meta-crtc.h"
 
 #define META_TYPE_CRTC_NATIVE (meta_crtc_native_get_type ())
+META_EXPORT_TEST
 G_DECLARE_DERIVABLE_TYPE (MetaCrtcNative, meta_crtc_native,
                           META, CRTC_NATIVE,
                           MetaCrtc)
@@ -28,12 +29,15 @@ struct _MetaCrtcNativeClass
 {
   MetaCrtcClass parent_class;
 
-  gboolean (* is_transform_handled) (MetaCrtcNative       *crtc_native,
-                                     MetaMonitorTransform  monitor_transform);
+  gboolean (* is_transform_handled) (MetaCrtcNative      *crtc_native,
+                                     MtkMonitorTransform  monitor_transform);
   gboolean (* is_hw_cursor_supported) (MetaCrtcNative *crtc_native);
+  int64_t (* get_deadline_evasion) (MetaCrtcNative *crtc_native);
 };
 
-gboolean meta_crtc_native_is_transform_handled (MetaCrtcNative       *crtc_native,
-                                                MetaMonitorTransform  transform);
+gboolean meta_crtc_native_is_transform_handled (MetaCrtcNative      *crtc_native,
+                                                MtkMonitorTransform  transform);
 
 gboolean meta_crtc_native_is_hw_cursor_supported (MetaCrtcNative *crtc_native);
+
+int64_t meta_crtc_native_get_deadline_evasion (MetaCrtcNative *crtc_native);
