@@ -23,8 +23,6 @@
 
 typedef struct _MetaKmsPageFlipData MetaKmsPageFlipData;
 
-typedef void (* MetaPageFlipDataFeedbackFunc) (MetaKmsPageFlipData *page_flip_data);
-
 MetaKmsPageFlipData * meta_kms_page_flip_data_new (MetaKmsImplDevice *impl_device,
                                                    MetaKmsCrtc       *crtc);
 
@@ -34,7 +32,6 @@ void meta_kms_page_flip_data_unref (MetaKmsPageFlipData *page_flip_data);
 
 void meta_kms_page_flip_data_add_listener (MetaKmsPageFlipData                 *page_flip_data,
                                            const MetaKmsPageFlipListenerVtable *vtable,
-                                           MetaKmsPageFlipListenerFlag          flags,
                                            GMainContext                        *main_context,
                                            gpointer                             user_data,
                                            GDestroyNotify                       destroy_notify);
@@ -55,9 +52,6 @@ void meta_kms_page_flip_data_mode_set_fallback_in_impl (MetaKmsPageFlipData *pag
 void meta_kms_page_flip_data_discard_in_impl (MetaKmsPageFlipData *page_flip_data,
                                               const GError        *error);
 
-void meta_kms_page_flip_data_take_error (MetaKmsPageFlipData *page_flip_data,
-                                         GError              *error);
-
 void meta_kms_page_flip_data_make_symbolic (MetaKmsPageFlipData *page_flip_data);
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaKmsPageFlipData, meta_kms_page_flip_data_unref)
