@@ -12,14 +12,9 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_KMS_TYPES_PRIVATE_H
-#define META_KMS_TYPES_PRIVATE_H
+#pragma once
 
 typedef struct _MetaKmsResultListener MetaKmsResultListener;
-
-#endif /* META_KMS_TYPES_PRIVATE_H */
