@@ -66,10 +66,21 @@ typedef struct _MetaKmsPlaneAssignment
   MetaKmsPlaneRotation rotation;
 
   struct {
+    gboolean has_update;
     gboolean is_valid;
     int x;
     int y;
   } cursor_hotspot;
+
+  struct {
+    gboolean has_update;
+    MetaKmsPlaneYCbCrColorEncoding value;
+  } color_encoding;
+
+  struct {
+    gboolean has_update;
+    MetaKmsPlaneYCbCrColorRange value;
+  } color_range;
 } MetaKmsPlaneAssignment;
 
 typedef struct _MetaKmsModeSet
@@ -109,15 +120,29 @@ typedef struct _MetaKmsConnectorUpdate
     gboolean has_update;
     MetaOutputHdrMetadata value;
   } hdr;
+
+  struct {
+    gboolean has_update;
+    MetaOutputRGBRange value;
+  } broadcast_rgb;
 } MetaKmsConnectorUpdate;
 
+typedef struct _MetaKmsCrtcUpdate
+{
+  MetaKmsCrtc *crtc;
+
+  struct {
+    gboolean has_update;
+    gboolean is_enabled;
+  } vrr;
+} MetaKmsCrtcUpdate;
+
 typedef struct _MetaKmsPageFlipListener
 {
   gatomicrefcount ref_count;
 
   MetaKmsCrtc *crtc;
   const MetaKmsPageFlipListenerVtable *vtable;
-  MetaKmsPageFlipListenerFlag flags;
   GMainContext *main_context;
   gpointer user_data;
   GDestroyNotify destroy_notify;
@@ -157,6 +182,12 @@ MetaKmsFeedback * meta_kms_feedback_new_failed (GList  *failed_planes,
 void meta_kms_plane_assignment_set_rotation (MetaKmsPlaneAssignment *plane_assignment,
                                              MetaKmsPlaneRotation    rotation);
 
+void meta_kms_plane_assignment_set_color_encoding (MetaKmsPlaneAssignment         *plane_assignment,
+                                                   MetaKmsPlaneYCbCrColorEncoding  color_encoding);
+
+void meta_kms_plane_assignment_set_color_range (MetaKmsPlaneAssignment      *plane_assignment,
+                                                MetaKmsPlaneYCbCrColorRange  range);
+
 META_EXPORT_TEST
 MetaKmsPlaneAssignment * meta_kms_update_get_primary_plane_assignment (MetaKmsUpdate *update,
                                                                        MetaKmsCrtc   *crtc);
@@ -177,6 +208,9 @@ GList * meta_kms_update_get_page_flip_listeners (MetaKmsUpdate *update);
 META_EXPORT_TEST
 GList * meta_kms_update_get_connector_updates (MetaKmsUpdate *update);
 
+META_EXPORT_TEST
+GList * meta_kms_update_get_crtc_updates (MetaKmsUpdate *update);
+
 META_EXPORT_TEST
 GList * meta_kms_update_get_crtc_color_updates (MetaKmsUpdate *update);
 
