@@ -12,14 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_RENDER_DEVICE_EGL_STREAM_H
-#define META_RENDER_DEVICE_EGL_STREAM_H
+#pragma once
 
 #include "backends/native/meta-render-device-private.h"
 
@@ -31,5 +28,3 @@ G_DECLARE_FINAL_TYPE (MetaRenderDeviceEglStream, meta_render_device_egl_stream,
 MetaRenderDeviceEglStream * meta_render_device_egl_stream_new (MetaBackend     *backend,
                                                                MetaDeviceFile  *device_file,
                                                                GError         **error);
-
-#endif /* META_RENDER_DEVICE_EGL_STREAM_H */
