@@ -654,13 +654,14 @@ cogl_context_from_renderer_native (MetaRendererNative *renderer_native)
 
 CoglFramebuffer *
 meta_renderer_native_create_dma_buf_framebuffer (MetaRendererNative  *renderer_native,
-                                                 int                  dmabuf_fd,
                                                  uint32_t             width,
                                                  uint32_t             height,
-                                                 uint32_t             stride,
-                                                 uint32_t             offset,
-                                                 uint64_t            *modifier,
                                                  uint32_t             drm_format,
+                                                 int                  n_planes,
+                                                 int                 *fds,
+                                                 uint32_t            *strides,
+                                                 uint32_t            *offsets,
+                                                 uint64_t            *modifiers,
                                                  GError             **error)
 {
   CoglContext *cogl_context =
@@ -671,8 +672,6 @@ meta_renderer_native_create_dma_buf_framebuffer (MetaRendererNative  *renderer_n
   EGLDisplay egl_display = cogl_renderer_egl->edpy;
   MetaEgl *egl = meta_renderer_native_get_egl (renderer_native);
   EGLImageKHR egl_image;
-  uint32_t strides[1];
-  uint32_t offsets[1];
   CoglPixelFormat cogl_format;
   CoglEglImageFlags flags;
   CoglTexture *cogl_tex;
@@ -683,18 +682,16 @@ meta_renderer_native_create_dma_buf_framebuffer (MetaRendererNative  *renderer_n
   g_assert (format_info);
   cogl_format = format_info->cogl_format;
 
-  strides[0] = stride;
-  offsets[0] = offset;
   egl_image = meta_egl_create_dmabuf_image (egl,
                                             egl_display,
                                             width,
                                             height,
                                             drm_format,
-                                            1 /* n_planes */,
-                                            &dmabuf_fd,
+                                            n_planes,
+                                            fds,
                                             strides,
                                             offsets,
-                                            modifier,
+                                            modifiers,
                                             error);
   if (egl_image == EGL_NO_IMAGE_KHR)
     return NULL;
@@ -752,12 +749,17 @@ static gboolean
 dummy_power_save_page_flip_cb (gpointer user_data)
 {
   MetaRendererNative *renderer_native = user_data;
+  g_autolist (GObject) old_list = NULL;
 
-  g_list_foreach (renderer_native->power_save_page_flip_onscreens,
+  old_list = g_steal_pointer (&renderer_native->power_save_page_flip_onscreens);
+
+  g_list_foreach (old_list,
                   (GFunc) meta_onscreen_native_dummy_power_save_page_flip,
                   NULL);
-  g_clear_list (&renderer_native->power_save_page_flip_onscreens,
-                g_object_unref);
+
+  if (renderer_native->power_save_page_flip_onscreens != NULL)
+    return G_SOURCE_CONTINUE;
+
   renderer_native->power_save_page_flip_source_id = 0;
 
   return G_SOURCE_REMOVE;
@@ -769,6 +771,9 @@ meta_renderer_native_queue_power_save_page_flip (MetaRendererNative *renderer_na
 {
   const unsigned int timeout_ms = 100;
 
+  if (g_list_find (renderer_native->power_save_page_flip_onscreens, onscreen))
+    return;
+
   if (!renderer_native->power_save_page_flip_source_id)
     {
       renderer_native->power_save_page_flip_source_id =
@@ -828,15 +833,13 @@ free_unused_gpu_datas (MetaRendererNative *renderer_native)
                                used_gpus);
 }
 
-static gboolean
+static void
 release_unused_gpus_idle (gpointer user_data)
 {
   MetaRendererNative *renderer_native = META_RENDERER_NATIVE (user_data);
 
   renderer_native->release_unused_gpus_idle_id = 0;
   free_unused_gpu_datas (renderer_native);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -851,7 +854,7 @@ old_onscreen_freed (gpointer  user_data,
   if (!renderer_native->release_unused_gpus_idle_id)
     {
       renderer_native->release_unused_gpus_idle_id =
-        g_idle_add (release_unused_gpus_idle, renderer_native);
+        g_idle_add_once (release_unused_gpus_idle, renderer_native);
     }
 }
 
@@ -966,6 +969,50 @@ meta_renderer_native_queue_mode_set_update (MetaRendererNative *renderer_native,
   meta_kms_update_free (new_kms_update);
 }
 
+static GArray *
+meta_renderer_native_query_drm_modifiers (CoglRenderer           *cogl_renderer,
+                                          CoglPixelFormat         format,
+                                          CoglDrmModifierFilter   filter,
+                                          GError                **error)
+{
+  CoglRendererEGL *cogl_renderer_egl = cogl_renderer->winsys;
+  MetaRendererNativeGpuData *renderer_gpu_data = cogl_renderer_egl->platform;
+  const MetaFormatInfo *format_info;
+  uint32_t drm_format;
+  MetaRenderDevice *render_device;
+
+  format_info = meta_format_info_from_cogl_format (format);
+  if (!format_info)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                   "Format %s not supported",
+                   cogl_pixel_format_to_string (format));
+      return NULL;
+    }
+
+  drm_format = format_info->drm_format;
+
+  render_device = renderer_gpu_data->render_device;
+  return meta_render_device_query_drm_modifiers (render_device, drm_format,
+                                                 filter, error);
+}
+
+static uint64_t
+meta_renderer_native_get_implicit_drm_modifier (CoglRenderer *renderer)
+{
+  return DRM_FORMAT_MOD_INVALID;
+}
+
+static void
+close_fds (int *fds,
+           int  n_fds)
+{
+  int i;
+
+  for (i = 0; i < n_fds; i++)
+    close (fds[i]);
+}
+
 static CoglDmaBufHandle *
 meta_renderer_native_create_dma_buf (CoglRenderer     *cogl_renderer,
                                      CoglPixelFormat   format,
@@ -986,12 +1033,16 @@ meta_renderer_native_create_dma_buf (CoglRenderer     *cogl_renderer,
         MetaRenderDevice *render_device;
         MetaDrmBufferFlags flags;
         g_autoptr (MetaDrmBuffer) buffer = NULL;
-        int dmabuf_fd;
-        uint32_t stride;
-        uint32_t offset;
+        uint64_t buffer_modifier;
+        int n_planes;
+        int *fds;
+        uint32_t *offsets;
+        uint32_t *strides;
+        uint64_t *plane_modifiers = NULL;
         uint32_t bpp;
         uint32_t drm_format;
-        CoglFramebuffer *dmabuf_fb;
+        int i;
+        g_autoptr (CoglFramebuffer) dmabuf_fb = NULL;
         CoglDmaBufHandle *dmabuf_handle;
         const MetaFormatInfo *format_info;
 
@@ -1016,52 +1067,61 @@ meta_renderer_native_create_dma_buf (CoglRenderer     *cogl_renderer,
         if (!buffer)
           return NULL;
 
-        dmabuf_fd = meta_drm_buffer_export_fd (buffer, error);
-        if (dmabuf_fd == -1)
-          return NULL;
-
-        stride = meta_drm_buffer_get_stride (buffer);
-        offset = meta_drm_buffer_get_offset (buffer, 0);
+        buffer_modifier = meta_drm_buffer_get_modifier (buffer);
         bpp = meta_drm_buffer_get_bpp (buffer);
-        if (n_modifiers)
-          {
-            uint64_t modifier = meta_drm_buffer_get_modifier (buffer);
 
-            dmabuf_fb =
-              meta_renderer_native_create_dma_buf_framebuffer (renderer_native,
-                                                               dmabuf_fd,
-                                                               width, height,
-                                                               stride,
-                                                               offset,
-                                                               &modifier,
-                                                               drm_format,
-                                                               error);
-          }
-        else
+        n_planes = meta_drm_buffer_get_n_planes (buffer);
+        fds = g_newa (int, n_planes);
+        offsets = g_newa (uint32_t, n_planes);
+        strides = g_newa (uint32_t, n_planes);
+
+        if (n_modifiers > 0)
+          plane_modifiers = g_newa (uint64_t, n_planes);
+
+        for (i = 0; i < n_planes; i++)
           {
-            dmabuf_fb =
-              meta_renderer_native_create_dma_buf_framebuffer (renderer_native,
-                                                               dmabuf_fd,
-                                                               width, height,
-                                                               stride,
-                                                               offset,
-                                                               NULL,
-                                                               drm_format,
-                                                               error);
+            fds[i] = meta_drm_buffer_export_fd_for_plane (buffer, i, error);
+            if (fds[i] == -1)
+              {
+                close_fds (fds, i);
+                return NULL;
+              }
+
+            offsets[i] = meta_drm_buffer_get_offset_for_plane (buffer, i);
+            strides[i] = meta_drm_buffer_get_stride_for_plane (buffer, i);
+            if (n_modifiers > 0)
+              plane_modifiers[i] = buffer_modifier;
           }
 
+        dmabuf_fb =
+          meta_renderer_native_create_dma_buf_framebuffer (renderer_native,
+                                                           width,
+                                                           height,
+                                                           drm_format,
+                                                           n_planes,
+                                                           fds,
+                                                           strides,
+                                                           offsets,
+                                                           plane_modifiers,
+                                                           error);
         if (!dmabuf_fb)
           {
-            close (dmabuf_fd);
+            close_fds (fds, n_planes);
             return NULL;
           }
 
         dmabuf_handle =
-          cogl_dma_buf_handle_new (dmabuf_fb, dmabuf_fd,
-                                   width, height, stride, offset, bpp,
+          cogl_dma_buf_handle_new (dmabuf_fb,
+                                   width, height,
+                                   format,
+                                   buffer_modifier,
+                                   n_planes,
+                                   fds,
+                                   strides,
+                                   offsets,
+                                   bpp,
                                    g_steal_pointer (&buffer),
                                    g_object_unref);
-        g_object_unref (dmabuf_fb);
         return dmabuf_handle;
       }
       break;
@@ -1255,7 +1315,7 @@ meta_renderer_native_create_offscreen (MetaRendererNative    *renderer_native,
                                              view_width, view_height,
                                              format);
     }
-  cogl_texture_set_auto_mipmap (tex, FALSE);
+  cogl_texture_2d_set_auto_mipmap (COGL_TEXTURE_2D (tex), FALSE);
 
   if (!cogl_texture_allocate (tex, error))
     {
@@ -1292,6 +1352,9 @@ get_native_cogl_winsys_vtable (CoglRenderer *cogl_renderer)
 
       vtable.renderer_connect = meta_renderer_native_connect;
       vtable.renderer_disconnect = meta_renderer_native_disconnect;
+      vtable.renderer_query_drm_modifiers = meta_renderer_native_query_drm_modifiers;
+      vtable.renderer_get_implicit_drm_modifier =
+        meta_renderer_native_get_implicit_drm_modifier;
       vtable.renderer_create_dma_buf = meta_renderer_native_create_dma_buf;
       vtable.renderer_is_dma_buf_supported =
         meta_renderer_native_is_dma_buf_supported;
@@ -1517,6 +1580,26 @@ detach_onscreens (MetaRenderer *renderer)
     }
 }
 
+static void
+discard_pending_swaps (MetaRenderer *renderer)
+{
+  GList *views = meta_renderer_get_views (renderer);;
+  GList *l;
+
+  for (l = views; l; l = l->next)
+    {
+      ClutterStageView *stage_view = l->data;
+      CoglFramebuffer *fb = clutter_stage_view_get_onscreen (stage_view);
+      CoglOnscreen *onscreen;
+
+      if (!COGL_IS_ONSCREEN (fb))
+        continue;
+
+      onscreen = COGL_ONSCREEN (fb);
+      meta_onscreen_native_discard_pending_swaps (onscreen);
+    }
+}
+
 static void
 meta_renderer_native_rebuild_views (MetaRenderer *renderer)
 {
@@ -1527,6 +1610,7 @@ meta_renderer_native_rebuild_views (MetaRenderer *renderer)
   MetaRendererClass *parent_renderer_class =
     META_RENDERER_CLASS (meta_renderer_native_parent_class);
 
+  discard_pending_swaps (renderer);
   meta_kms_discard_pending_page_flips (kms);
   g_hash_table_remove_all (renderer_native->mode_set_updates);
 
@@ -1830,6 +1914,33 @@ maybe_restore_cogl_egl_api (MetaRendererNative *renderer_native)
   cogl_renderer_bind_api (cogl_renderer);
 }
 
+static void
+set_default_secondary_gpu_copy_mode (MetaRendererNativeGpuData *gpu_data)
+{
+  const char *copy_mode;
+
+  gpu_data->secondary.copy_mode =
+    META_SHARED_FRAMEBUFFER_COPY_MODE_SECONDARY_GPU;
+
+  copy_mode = getenv ("MUTTER_DEBUG_MULTI_GPU_FORCE_COPY_MODE");
+  if (!copy_mode || *copy_mode == '\0')
+    return;
+
+  if (strcmp (copy_mode, "primary-gpu-gpu") == 0)
+    {
+      gpu_data->secondary.copy_mode = META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY;
+    }
+  else if (strcmp (copy_mode, "primary-gpu-cpu") == 0)
+    {
+      gpu_data->secondary.copy_mode = META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY;
+      gpu_data->secondary.copy_mode_primary_force_cpu = TRUE;
+    }
+  else if (strcmp (copy_mode, "zero-copy") == 0)
+    {
+      gpu_data->secondary.copy_mode = META_SHARED_FRAMEBUFFER_COPY_MODE_ZERO;
+    }
+}
+
 static gboolean
 init_secondary_gpu_data_gpu (MetaRendererNativeGpuData *renderer_gpu_data,
                              GError                   **error)
@@ -1905,7 +2016,7 @@ init_secondary_gpu_data_gpu (MetaRendererNativeGpuData *renderer_gpu_data,
 
   renderer_gpu_data->secondary.egl_context = egl_context;
   renderer_gpu_data->secondary.egl_config = egl_config;
-  renderer_gpu_data->secondary.copy_mode = META_SHARED_FRAMEBUFFER_COPY_MODE_SECONDARY_GPU;
+  set_default_secondary_gpu_copy_mode (renderer_gpu_data);
 
   renderer_gpu_data->secondary.has_EGL_EXT_image_dma_buf_import_modifiers =
     meta_egl_has_extensions (egl, egl_display, NULL,
@@ -2438,16 +2549,6 @@ initable_iface_init (GInitableIface *initable_iface)
   initable_iface->init = meta_renderer_native_initable_init;
 }
 
-static void
-meta_renderer_native_dispose (GObject *object)
-{
-  MetaRendererNative *renderer_native = META_RENDERER_NATIVE (object);
-
-  clear_detached_onscreens (renderer_native);
-
-  G_OBJECT_CLASS (meta_renderer_native_parent_class)->dispose (object);
-}
-
 static void
 meta_renderer_native_finalize (GObject *object)
 {
@@ -2463,6 +2564,7 @@ meta_renderer_native_finalize (GObject *object)
 
   g_clear_handle_id (&renderer_native->release_unused_gpus_idle_id,
                      g_source_remove);
+  clear_detached_onscreens (renderer_native);
 
   g_hash_table_destroy (renderer_native->gpu_datas);
   g_clear_object (&renderer_native->gles3);
@@ -2511,7 +2613,6 @@ meta_renderer_native_class_init (MetaRendererNativeClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   MetaRendererClass *renderer_class = META_RENDERER_CLASS (klass);
 
-  object_class->dispose = meta_renderer_native_dispose;
   object_class->finalize = meta_renderer_native_finalize;
   object_class->constructed = meta_renderer_native_constructed;
 
