@@ -52,8 +52,7 @@ int64_t meta_compositor_monotonic_to_high_res_xserver_time (MetaCompositor *comp
                                                             int64_t         monotonic_time_us);
 
 void meta_compositor_flash_window (MetaCompositor *compositor,
-                                   MetaWindow     *window,
-                                   int             n_flashes);
+                                   MetaWindow     *window);
 
 MetaCloseDialog * meta_compositor_create_close_dialog (MetaCompositor *compositor,
                                                        MetaWindow     *window);
@@ -67,12 +66,7 @@ gboolean meta_compositor_is_unredirect_inhibited (MetaCompositor *compositor);
 
 MetaDisplay * meta_compositor_get_display (MetaCompositor *compositor);
 
-MetaBackend * meta_compositor_get_backend (MetaCompositor *compositor);
-
 MetaWindowActor * meta_compositor_get_top_window_actor (MetaCompositor *compositor);
-
-ClutterStage * meta_compositor_get_stage (MetaCompositor *compositor);
-
 gboolean meta_compositor_is_switching_workspace (MetaCompositor *compositor);
 
 gboolean meta_compositor_drag_window (MetaCompositor       *compositor,
@@ -81,7 +75,8 @@ gboolean meta_compositor_drag_window (MetaCompositor       *compositor,
                                       ClutterInputDevice   *device,
                                       ClutterEventSequence *sequence,
                                       uint32_t              timestamp,
-                                      graphene_point_t     *pos_hint);
+                                      graphene_point_t     *pos_hint,
+                                      ClutterActor         *grab_actor);
 
 MetaWindowDrag * meta_compositor_get_current_window_drag (MetaCompositor *compositor);
 
@@ -146,8 +141,7 @@ void meta_compositor_sync_stack (MetaCompositor *compositor,
                                  GList          *stack);
 
 void meta_compositor_flash_display (MetaCompositor *compositor,
-                                    MetaDisplay    *display,
-                                    int             n_flashes);
+                                    MetaDisplay    *display);
 
 void meta_compositor_show_tile_preview (MetaCompositor *compositor,
                                         MetaWindow     *window,
