@@ -42,6 +42,7 @@
 
 #ifdef HAVE_NATIVE_BACKEND
 #include "backends/native/meta-backend-native.h"
+#include "backends/native/meta-backend-native-types.h"
 #endif
 
 #if defined (HAVE_X11) && defined (HAVE_WAYLAND)
@@ -348,8 +349,10 @@ meta_context_main_get_x11_display_policy (MetaContext *context)
 #ifdef HAVE_WAYLAND
       if (context_main->options.no_x11)
         return META_X11_DISPLAY_POLICY_DISABLED;
+#ifdef HAVE_LOGIND
       else if (sd_pid_get_user_unit (0, &unit) < 0)
         return META_X11_DISPLAY_POLICY_MANDATORY;
+#endif
       else
         return META_X11_DISPLAY_POLICY_ON_DEMAND;
 #else /* HAVE_WAYLAND */
@@ -663,8 +666,6 @@ meta_context_main_add_option_entries (MetaContextMain *context_main)
       N_("Run as a nested compositor"),
       NULL
     },
-#endif
-#ifdef HAVE_XWAYLAND
     {
       "no-x11", 0, 0, G_OPTION_ARG_NONE,
       &context_main->options.no_x11,
