@@ -12,16 +12,13 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Carlos Garnacho <carlosg@gnome.org>
  */
 
-#ifndef META_SELECTION_PRIVATE_H
-#define META_SELECTION_PRIVATE_H
+#pragma once
 
 #include "meta/meta-selection.h"
 
@@ -30,5 +27,3 @@ MetaSelectionSource *
                                     MetaSelectionType  selection_type);
 
 MetaDisplay * meta_selection_get_display (MetaSelection *selection);
-
-#endif /* META_SELECTION_PRIVATE_H */
