@@ -844,8 +844,6 @@ meta_window_place (MetaWindow        *window,
                    MetaPlaceFlag      flags,
                    int                x,
                    int                y,
-                   int                new_width,
-                   int                new_height,
                    int               *new_x,
                    int               *new_y)
 {
@@ -856,9 +854,7 @@ meta_window_place (MetaWindow        *window,
   MetaLogicalMonitor *logical_monitor = NULL;
   gboolean place_centered = FALSE;
 
-  meta_topic (META_DEBUG_PLACEMENT,
-              "Placing window (%s), width: %d, height: %d",
-              window->desc, new_width, new_height);
+  meta_topic (META_DEBUG_PLACEMENT, "Placing window %s", window->desc);
 
   g_return_if_fail (!window->placement.rule);
 
@@ -961,8 +957,9 @@ meta_window_place (MetaWindow        *window,
 
       if (parent)
         {
-          MtkRectangle parent_frame_rect;
+          MtkRectangle frame_rect, parent_frame_rect;
 
+          meta_window_get_frame_rect (window, &frame_rect);
           meta_window_get_frame_rect (parent, &parent_frame_rect);
 
           y = parent_frame_rect.y;
@@ -970,12 +967,12 @@ meta_window_place (MetaWindow        *window,
           /* center of parent */
           x = parent_frame_rect.x + parent_frame_rect.width / 2;
           /* center of child over center of parent */
-          x -= new_width / 2;
+          x -= frame_rect.width / 2;
 
           /* "visually" center window over parent, leaving twice as
            * much space below as on top.
            */
-          y += (parent_frame_rect.height - new_height) / 3;
+          y += (parent_frame_rect.height - frame_rect.height) / 3;
 
           meta_topic (META_DEBUG_PLACEMENT,
                       "Centered window %s over transient parent",
@@ -999,13 +996,15 @@ meta_window_place (MetaWindow        *window,
     {
       /* Center on current monitor */
       MtkRectangle work_area;
+      MtkRectangle frame_rect;
 
       meta_window_get_work_area_for_logical_monitor (window,
                                                      logical_monitor,
                                                      &work_area);
+      meta_window_get_frame_rect (window, &frame_rect);
 
-      x = work_area.x + (work_area.width - new_width) / 2;
-      y = work_area.y + (work_area.height - new_height) / 2;
+      x = work_area.x + (work_area.width - frame_rect.width) / 2;
+      y = work_area.y + (work_area.height - frame_rect.height) / 2;
 
       meta_topic (META_DEBUG_PLACEMENT, "Centered window %s on monitor %d",
                   window->desc, logical_monitor->number);
@@ -1024,6 +1023,31 @@ meta_window_place (MetaWindow        *window,
         find_next_cascade (window, windows, &x, &y, place_centered);
     }
 
+  /* Maximize windows if they are too big for their work area (bit of
+   * a hack here). Assume undecorated windows probably don't intend to
+   * be maximized.
+   */
+  if (window->has_maximize_func && window->decorated &&
+      !meta_window_is_fullscreen (window))
+    {
+      MtkRectangle workarea;
+      MtkRectangle frame_rect;
+
+      meta_window_get_work_area_for_logical_monitor (window,
+                                                     logical_monitor,
+                                                     &workarea);
+      meta_window_get_frame_rect (window, &frame_rect);
+
+      /* If the window is bigger than the screen, then automaximize.  Do NOT
+       * auto-maximize the directions independently.  See #419810.
+       */
+      if (frame_rect.width >= workarea.width && frame_rect.height >= workarea.height)
+        {
+          window->maximize_horizontally_after_placement = TRUE;
+          window->maximize_vertically_after_placement = TRUE;
+        }
+    }
+
   /* If the window is being denied focus and isn't a transient of the
    * focus window, we do NOT want it to overlap with the focus window
    * if at all possible.  This is guaranteed to only be called if the
