@@ -19,21 +19,18 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_PLACE_H
-#define META_PLACE_H
+#pragma once
 
-#include "core/frame.h"
 #include "core/window-private.h"
 
 void meta_window_process_placement (MetaWindow        *window,
                                     MetaPlacementRule *placement_rule,
-                                    int               *x,
-                                    int               *y);
+                                    int               *rel_x,
+                                    int               *rel_y);
 
-void meta_window_place (MetaWindow *window,
-                        int         x,
-                        int         y,
-                        int        *new_x,
-                        int        *new_y);
-
-#endif
+void meta_window_place (MetaWindow    *window,
+                        MetaPlaceFlag  place_flags,
+                        int            x,
+                        int            y,
+                        int           *new_x,
+                        int           *new_y);
