@@ -12,15 +12,10 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef PREFS_PRIVATE_H
-#define PREFS_PRIVATE_H
+#pragma once
 
 void meta_prefs_init (void);
-
-#endif /* PREFS_PRIVATE_H */
