@@ -77,8 +77,8 @@ static GList *listeners = NULL;
 static GHashTable *settings_schemas;
 
 static ClutterModifierType mouse_button_mods = CLUTTER_MOD1_MASK;
-static MetaKeyCombo overlay_key_combo = { 0, 0, 0 };
-static MetaKeyCombo locate_pointer_key_combo = { 0, 0, 0 };
+static MetaKeyCombo overlay_key_combos[2] = { 0 };
+static MetaKeyCombo locate_pointer_key_combos[2] = { 0 };
 static GDesktopFocusMode focus_mode = G_DESKTOP_FOCUS_MODE_CLICK;
 static GDesktopFocusNewWindows focus_new_windows = G_DESKTOP_FOCUS_NEW_WINDOWS_SMART;
 static gboolean raise_on_click = TRUE;
@@ -1137,8 +1137,8 @@ maybe_give_disable_workarounds_warning (void)
     {
       first_disable = FALSE;
 
-      meta_warning ("Workarounds for broken applications disabled. "
-                    "Some applications may not behave properly.");
+      g_warning ("Workarounds for broken applications disabled. "
+                 "Some applications may not behave properly.");
     }
 }
 
@@ -1227,12 +1227,10 @@ mouse_button_mods_handler (GVariant *value,
 
   if (!string_value || !meta_parse_modifier (string_value, &mods))
     {
-      meta_topic (META_DEBUG_KEYBINDINGS,
-                  "Failed to parse new GSettings value");
-
-      meta_warning ("\"%s\" found in configuration database is "
-                    "not a valid value for mouse button modifier",
-                    string_value);
+      g_warning ("Failed to parse new GSettings value: "
+                 "\"%s\" found in configuration database is "
+                 "not a valid value for mouse button modifier",
+                 string_value);
 
       return FALSE;
     }
@@ -1467,33 +1465,56 @@ button_layout_handler (GVariant *value,
   return TRUE;
 }
 
+static gboolean
+parse_special_key (const gchar  *string_value,
+                   MetaKeyCombo  combos[2])
+{
+  g_autofree gchar *string_value_l = NULL;
+  g_autofree gchar *string_value_r = NULL;
+
+  if (meta_parse_accelerator (string_value, &combos[0]))
+    return TRUE;
+
+  string_value_l = g_strconcat (string_value, "_L", NULL);
+  if (!meta_parse_accelerator (string_value_l, &combos[0]))
+    return FALSE;
+
+  string_value_r = g_strconcat (string_value, "_R", NULL);
+  if (!meta_parse_accelerator (string_value_r, &combos[1]))
+    return FALSE;
+
+  return TRUE;
+}
+
 static gboolean
 overlay_key_handler (GVariant *value,
                      gpointer *result,
                      gpointer  data)
 {
-  MetaKeyCombo combo;
+  MetaKeyCombo combos[2] = { 0 };
   const gchar *string_value;
+  int i;
 
   *result = NULL; /* ignored */
   string_value = g_variant_get_string (value, NULL);
 
-  if (string_value && meta_parse_accelerator (string_value, &combo))
-    ;
-  else
+  if (!string_value || !parse_special_key (string_value, combos))
     {
       meta_topic (META_DEBUG_KEYBINDINGS,
                   "Failed to parse value for overlay-key");
       return FALSE;
     }
 
-  combo.modifiers = 0;
-
-  if (overlay_key_combo.keysym != combo.keysym ||
-      overlay_key_combo.keycode != combo.keycode)
+  for (i = 0; i < G_N_ELEMENTS (combos); i++)
     {
-      overlay_key_combo = combo;
-      queue_changed (META_PREF_KEYBINDINGS);
+      combos[i].modifiers = 0;
+
+      if (overlay_key_combos[i].keysym != combos[i].keysym ||
+          overlay_key_combos[i].keycode != combos[i].keycode)
+        {
+          overlay_key_combos[i] = combos[i];
+          queue_changed (META_PREF_KEYBINDINGS);
+        }
     }
 
   return TRUE;
@@ -1504,26 +1525,30 @@ locate_pointer_key_handler (GVariant *value,
                             gpointer *result,
                             gpointer  data)
 {
-  MetaKeyCombo combo;
+  MetaKeyCombo combos[2] = { 0 };
   const gchar *string_value;
+  int i;
 
   *result = NULL; /* ignored */
   string_value = g_variant_get_string (value, NULL);
 
-  if (!string_value || !meta_parse_accelerator (string_value, &combo))
+  if (!string_value || !parse_special_key (string_value, combos))
     {
       meta_topic (META_DEBUG_KEYBINDINGS,
                   "Failed to parse value for locate-pointer-key");
       return FALSE;
     }
 
-  combo.modifiers = 0;
-
-  if (locate_pointer_key_combo.keysym != combo.keysym ||
-      locate_pointer_key_combo.keycode != combo.keycode)
+  for (i = 0; i < G_N_ELEMENTS (combos); i++)
     {
-      locate_pointer_key_combo = combo;
-      queue_changed (META_PREF_KEYBINDINGS);
+      combos[i].modifiers = 0;
+
+      if (locate_pointer_key_combos[i].keysym != combos[i].keysym ||
+          locate_pointer_key_combos[i].keycode != combos[i].keycode)
+        {
+          locate_pointer_key_combos[i] = combos[i];
+          queue_changed (META_PREF_KEYBINDINGS);
+        }
     }
 
   return TRUE;
@@ -1733,7 +1758,8 @@ init_bindings (void)
   pref = g_new0 (MetaKeyPref, 1);
   pref->name = g_strdup ("overlay-key");
   pref->action = META_KEYBINDING_ACTION_OVERLAY_KEY;
-  pref->combos = g_slist_prepend (pref->combos, &overlay_key_combo);
+  pref->combos = g_slist_prepend (pref->combos, &overlay_key_combos[0]);
+  pref->combos = g_slist_prepend (pref->combos, &overlay_key_combos[1]);
   pref->builtin = 1;
 
   g_hash_table_insert (key_bindings, g_strdup (pref->name), pref);
@@ -1741,7 +1767,8 @@ init_bindings (void)
   pref = g_new0 (MetaKeyPref, 1);
   pref->name = g_strdup ("locate-pointer-key");
   pref->action = META_KEYBINDING_ACTION_LOCATE_POINTER_KEY;
-  pref->combos = g_slist_prepend (pref->combos, &locate_pointer_key_combo);
+  pref->combos = g_slist_prepend (pref->combos, &locate_pointer_key_combos[0]);
+  pref->combos = g_slist_prepend (pref->combos, &locate_pointer_key_combos[1]);
   pref->builtin = 1;
 
   g_hash_table_insert (key_bindings, g_strdup (pref->name), pref);
@@ -1770,10 +1797,10 @@ update_binding (MetaKeyPref *binding,
 
       if (!meta_parse_accelerator (strokes[i], combo))
         {
-          meta_topic (META_DEBUG_KEYBINDINGS,
-                      "Failed to parse new GSettings value");
-          meta_warning ("\"%s\" found in configuration database is not a valid value for keybinding \"%s\"",
-                        strokes[i], binding->name);
+          g_warning ("Failed to parse new GSettings value: "
+                     "\"%s\" found in configuration database is not a valid "
+                     "value for keybinding \"%s\"",
+                     strokes[i], binding->name);
 
           g_free (combo);
 
@@ -1940,7 +1967,7 @@ meta_prefs_add_keybinding (const char           *name,
 
   if (g_hash_table_lookup (key_bindings, name))
     {
-      meta_warning ("Trying to re-add keybinding \"%s\".", name);
+      g_warning ("Trying to re-add keybinding \"%s\".", name);
       return FALSE;
     }
 
@@ -1990,13 +2017,13 @@ meta_prefs_remove_keybinding (const char *name)
   pref = g_hash_table_lookup (key_bindings, name);
   if (!pref)
     {
-      meta_warning ("Trying to remove non-existent keybinding \"%s\".", name);
+      g_warning ("Trying to remove non-existent keybinding \"%s\".", name);
       return FALSE;
     }
 
   if (pref->builtin)
     {
-      meta_warning ("Trying to remove builtin keybinding \"%s\".", name);
+      g_warning ("Trying to remove builtin keybinding \"%s\".", name);
       return FALSE;
     }
 
@@ -2017,15 +2044,17 @@ meta_prefs_get_keybindings (void)
 }
 
 void
-meta_prefs_get_overlay_binding (MetaKeyCombo *combo)
+meta_prefs_get_overlay_bindings (MetaKeyCombo combos[2])
 {
-  *combo = overlay_key_combo;
+  combos[0] = overlay_key_combos[0];
+  combos[1] = overlay_key_combos[1];
 }
 
 void
-meta_prefs_get_locate_pointer_binding (MetaKeyCombo *combo)
+meta_prefs_get_locate_pointer_bindings (MetaKeyCombo combos[2])
 {
-  *combo = locate_pointer_key_combo;
+  combos[0] = locate_pointer_key_combos[0];
+  combos[1] = locate_pointer_key_combos[1];
 }
 
 gboolean
