@@ -103,8 +103,7 @@ static void
 gesture_relationship_cancel_on_recognize (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterGesture *gesture_1 = CLUTTER_GESTURE (g_object_new (TEST_TYPE_GESTURE, "name", "gesture-1", NULL));
@@ -162,8 +161,7 @@ static void
 gesture_relationship_simple (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterGesture *gesture_1 = CLUTTER_GESTURE (g_object_new (TEST_TYPE_GESTURE, "name", "gesture-1", NULL));
@@ -224,8 +222,7 @@ static void
 gesture_relationship_two_points (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterGesture *gesture_1 = CLUTTER_GESTURE (g_object_new (TEST_TYPE_GESTURE, "name", "gesture-1", NULL));
@@ -273,8 +270,7 @@ static void
 gesture_relationship_two_points_two_actors (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterActor *second_actor = clutter_actor_new ();
@@ -342,8 +338,7 @@ static void
 gesture_relationship_claim_new_sequence_while_already_recognizing (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterGesture *gesture_1 = CLUTTER_GESTURE (g_object_new (TEST_TYPE_GESTURE, "name", "gesture-1", NULL));
@@ -440,8 +435,7 @@ static void
 gesture_relationship_claim_new_sequence_while_already_recognizing_2 (void)
 {
   ClutterActor *stage = clutter_test_get_stage ();
-  ClutterSeat *seat =
-    clutter_backend_get_default_seat (clutter_get_default_backend ());
+  ClutterSeat *seat = clutter_test_get_default_seat ();
   g_autoptr (ClutterVirtualInputDevice) virtual_pointer = NULL;
   int64_t now_us;
   ClutterActor *second_actor = clutter_actor_new ();
