@@ -47,7 +47,10 @@ create_primitives (CoglPrimitive *primitives[2])
                                            vertex_data);
   cogl_primitive_set_n_vertices (primitives[0], 4);
 
-  primitives[1] = cogl_primitive_copy (primitives[0]);
+  primitives[1] = cogl_primitive_new_p2c4 (test_ctx,
+                                           COGL_VERTICES_MODE_TRIANGLE_STRIP,
+                                           G_N_ELEMENTS (vertex_data),
+                                           vertex_data);
   cogl_primitive_set_first_vertex (primitives[1], 4);
   cogl_primitive_set_n_vertices (primitives[1], 4);
 }
