@@ -292,7 +292,8 @@ verify_screen_cast_content (const char               *ref_test_name,
   test_seq_no_string = g_strdup_printf ("%d", test_seq_no);
   reftest_flags = META_REFTEST_FLAG_NONE;
   subprocess =
-    meta_launch_test_executable ("mutter-cursor-tests-screen-cast-client",
+    meta_launch_test_executable (G_SUBPROCESS_FLAGS_NONE,
+                                 "mutter-cursor-tests-screen-cast-client",
                                  ref_test_name,
                                  test_seq_no_string,
                                  cursor_mode_to_string (cursor_mode),
@@ -418,7 +419,7 @@ meta_test_native_cursor_scaling (void)
   };
   int i;
 
-  cursor = META_CURSOR_MOVE_OR_RESIZE_WINDOW;
+  cursor = META_CURSOR_MOVE;
   meta_display_set_cursor (display, cursor);
   virtual_pointer = clutter_seat_create_virtual_device (seat,
                                                         CLUTTER_POINTER_DEVICE);
@@ -534,7 +535,7 @@ meta_test_native_cursor_cropping (void)
 
       test_client_cursor (view,
                           CURSOR_SCALE_METHOD_VIEWPORT_CROPPED,
-                          META_CURSOR_MOVE_OR_RESIZE_WINDOW,
+                          META_CURSOR_MOVE,
                           MTK_MONITOR_TRANSFORM_NORMAL,
                           ref_test_name, 0,
                           meta_ref_test_determine_ref_test_flag ());
@@ -628,7 +629,7 @@ meta_test_native_cursor_transform (void)
                           meta_ref_test_determine_ref_test_flag ());
       test_client_cursor (view,
                           CURSOR_SCALE_METHOD_VIEWPORT_CROPPED,
-                          META_CURSOR_MOVE_OR_RESIZE_WINDOW,
+                          META_CURSOR_MOVE,
                           test_cases[i].transform,
                           ref_test_name, 2,
                           meta_ref_test_determine_ref_test_flag ());
