@@ -82,6 +82,15 @@ ensure_gsettings_memory_backend (void)
                  G_TYPE_FROM_INSTANCE (default_backend));
 }
 
+static void
+ensure_xcursor_path (void)
+{
+  g_autofree char *xcursor_path = NULL;
+
+  xcursor_path = g_test_build_filename (G_TEST_DIST, "xcursors", NULL);
+  g_setenv ("XCURSOR_PATH", xcursor_path, TRUE);
+}
+
 static void
 meta_context_test_finalize (GObject *object)
 {
@@ -125,6 +134,7 @@ meta_context_test_configure (MetaContext   *context,
   meta_context_set_plugin_gtype (context, META_TYPE_TEST_SHELL);
 
   ensure_gsettings_memory_backend ();
+  ensure_xcursor_path ();
 
   return TRUE;
 }
@@ -283,7 +293,7 @@ run_tests_idle (gpointer user_data)
   if (g_signal_has_handler_pending (context, signals[RUN_TESTS], 0, TRUE))
     {
       g_signal_emit (context, signals[RUN_TESTS], 0, &ret);
-      g_assert (ret == 1 || ret == 0);
+      g_assert_true (ret == 1 || ret == 0);
     }
   else
     {
@@ -325,10 +335,8 @@ meta_context_test_run_tests (MetaContextTest  *context_test,
       if ((flags & META_TEST_RUN_FLAG_CAN_SKIP) &&
           ((g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND) &&
             strstr (error->message, "No GPUs found")) ||
-           (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_DBUS_ERROR) &&
-            strstr (error->message, "Could not take control")) ||
-           (g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD) &&
-            strstr (error->message, "Could not take control"))))
+           (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_FAILED) &&
+            strstr (error->message, "Native backend mode needs to be session controller"))))
         {
           g_printerr ("Test skipped: %s\n", error->message);
           return 77;
