@@ -24,3 +24,10 @@ META_EXPORT
 G_DECLARE_FINAL_TYPE (MetaTestShell, meta_test_shell,
                       META, TEST_SHELL,
                       MetaPlugin)
+
+META_EXPORT
+void meta_test_shell_set_background_color (MetaTestShell *test_shell,
+                                           CoglColor      color);
+
+META_EXPORT
+void meta_test_shell_disable_animations (MetaTestShell *test_shell);
