@@ -7,7 +7,6 @@ import fcntl
 import subprocess
 import getpass
 import argparse
-import logind_helpers
 import tempfile
 import select
 import socket
@@ -27,15 +26,6 @@ class MultiOrderedDict(OrderedDict):
         else:
             super(OrderedDict, self).__setitem__(key, value)
 
-
-def escape_object_path(path):
-    b = bytearray()
-    b.extend(path.encode())
-    path = Gio.dbus_escape_object_path_bytestring(b)
-    if path[0].isdigit():
-        path = "_{0:02x}{1}".format(ord(path[0]), path[1:])
-    return os.path.basename(path)
-
 def get_subprocess_stdout():
     if os.getenv('META_DBUS_RUNNER_VERBOSE') == '1':
         return sys.stderr
@@ -49,20 +39,14 @@ class MutterDBusRunner(DBusTestCase):
             return os.path.join(os.path.dirname(__file__), 'dbusmock-templates')
 
     @classmethod
-    def setUpClass(klass, enable_kvm=False, launch=[], bind_sockets=False):
+    def setUpClass(klass, launch=[], bind_sockets=False):
         klass.templates_dirs = [klass.__get_templates_dir()]
 
         klass.mocks = OrderedDict()
 
-        klass.host_system_bus_address = os.getenv('DBUS_SYSTEM_BUS_ADDRESS')
-        if klass.host_system_bus_address is None:
-            klass.host_system_bus_address = 'unix:path=/run/dbus/system_bus_socket'
-
-        try:
-            dbus.bus.BusConnection(klass.host_system_bus_address)
-            klass.has_host_system_bus = True
-        except:
-            klass.has_host_system_bus = False
+        host_system_bus_address = os.getenv('DBUS_SYSTEM_BUS_ADDRESS')
+        if host_system_bus_address is None:
+            host_system_bus_address = 'unix:path=/run/dbus/system_bus_socket'
 
         print('Starting D-Bus daemons (session & system)...', file=sys.stderr)
         DBusTestCase.setUpClass()
@@ -88,12 +72,13 @@ class MutterDBusRunner(DBusTestCase):
         klass.start_from_local_template('gsd-color')
         klass.start_from_local_template('rtkit')
         klass.start_from_local_template('screensaver')
+        klass.start_from_local_template('logind', {
+            'host_system_bus_address': host_system_bus_address,
+        })
 
         klass.system_bus_con = klass.get_dbus(system_bus=True)
         klass.session_bus_con = klass.get_dbus(system_bus=False)
 
-        klass.init_logind(enable_kvm)
-
         if klass.session_bus_con.name_has_owner('org.gnome.Mutter.DisplayConfig'):
             raise Exception(
                 'org.gnome.Mutter.DisplayConfig already has owner on the session bus, bailing')
@@ -133,15 +118,6 @@ class MutterDBusRunner(DBusTestCase):
         template = klass.find_template(template_file_name)
         return klass.start_from_template(template, params, system_bus=system_bus)
 
-    @classmethod
-    def start_from_template_managed(klass, template):
-        klass.mock_obj.StartFromTemplate(template)
-
-    @classmethod
-    def start_from_local_template_managed(klass, template_file_name):
-        template = klass.find_template(template_file_name)
-        klass.mock_obj.StartFromLocalTemplate(template)
-
     @classmethod
     def start_from_class(klass, mock_class, params={}):
         mock_server = \
@@ -158,116 +134,6 @@ class MutterDBusRunner(DBusTestCase):
         mocks = (mock_server, mock_obj)
         return mocks
 
-    def wrap_logind_call(call):
-        code = \
-f'''
-import os
-import sys
-
-sys.path.insert(0, '{os.path.dirname(__file__)}')
-import logind_helpers
-
-{call}
-'''
-        return code
-
-    @classmethod
-    def forward_to_host(klass, object_path, interface, method, in_type, out_type):
-        proxy = klass.system_bus_con.get_object('org.freedesktop.login1',
-                                                object_path)
-        proxy.AddMethod(interface, method, in_type, out_type,
-f'''
-import os
-import sys
-
-sys.path.insert(0, '{os.path.dirname(__file__)}')
-import logind_helpers
-
-ret = logind_helpers.call_host('{klass.host_system_bus_address}',
-                               '{object_path}',
-                               '{interface}',
-                               '{method}',
-                               '{in_type}',
-                               args)
-''')
-
-    @classmethod
-    def init_logind_forward(klass, session_path, seat_path):
-        klass.forward_to_host(session_path, 'org.freedesktop.login1.Session',
-                              'TakeDevice',
-                              'uu', 'hb')
-        klass.forward_to_host(session_path, 'org.freedesktop.login1.Session',
-                              'ReleaseDevice',
-                              'uu', '')
-        klass.forward_to_host(session_path, 'org.freedesktop.login1.Session',
-                              'TakeDevice',
-                              'uu', 'hb')
-        klass.forward_to_host(session_path, 'org.freedesktop.login1.Session',
-                              'TakeControl',
-                              'b', '')
-        klass.forward_to_host(seat_path, 'org.freedesktop.login1.Seat',
-                              'SwitchTo',
-                              'u', '')
-
-    @classmethod
-    def init_logind_kvm(klass, session_path):
-        session_obj = klass.system_bus_con.get_object('org.freedesktop.login1', session_path)
-        session_obj.AddMethod('org.freedesktop.login1.Session',
-                              'TakeDevice',
-                              'uu', 'hb',
-                              klass.wrap_logind_call(
-f'''
-major = args[0]
-minor = args[1]
-ret = logind_helpers.open_file_direct(major, minor)
-'''))
-        session_obj.AddMethods('org.freedesktop.login1.Session', [
-            ('ReleaseDevice', 'uu', '', ''),
-            ('TakeControl', 'b', '', ''),
-        ])
-
-
-    @classmethod
-    def find_host_session_name(klass):
-        if 'XDG_SESSION_ID' in os.environ:
-            return escape_object_path(os.environ['XDG_SESSION_ID'])
-
-        bus = dbus.bus.BusConnection(klass.host_system_bus_address)
-        session_auto_proxy = bus.get_object('org.freedesktop.login1',
-                                            '/org/freedesktop/login1/session/auto')
-        props = dbus.Interface(session_auto_proxy,
-                               dbus_interface='org.freedesktop.DBus.Properties')
-        session_id = props.Get('org.freedesktop.login1.Session', 'Id')
-        manager_proxy = bus.get_object('org.freedesktop.login1',
-                                       '/org/freedesktop/login1')
-        manager = dbus.Interface(manager_proxy,
-                                 dbus_interface='org.freedesktop.login1.Manager')
-        session_path = manager.GetSession(session_id)
-        return os.path.basename(session_path)
-
-    @classmethod
-    def init_logind(klass, enable_kvm):
-        logind = klass.start_from_template('logind')
-
-        [p_mock, obj] = logind
-
-        mock_iface = 'org.freedesktop.DBus.Mock'
-        seat_path = obj.AddSeat('seat0', dbus_interface=mock_iface)
-        session_name = 'dummy'
-        if klass.has_host_system_bus:
-            session_name = klass.find_host_session_name()
-
-        session_path = obj.AddSession(session_name, 'seat0',
-                                      dbus.types.UInt32(os.getuid()),
-                                      getpass.getuser(),
-                                      True,
-                                      dbus_interface=mock_iface)
-
-        if enable_kvm:
-            klass.init_logind_kvm(session_path)
-        elif klass.has_host_system_bus:
-            klass.init_logind_forward(session_path, seat_path)
-
     @classmethod
     def add_template_dir(klass, templates_dir):
         klass.templates_dirs += [templates_dir]
@@ -325,21 +191,16 @@ ret = logind_helpers.open_file_direct(major, minor)
     @classmethod
     def get_pipewire_socket_names(klass):
         pipewire_socket_unit = '/usr/lib/systemd/user/pipewire.socket'
-        if os.path.exists(pipewire_socket_unit):
-            config = configparser.ConfigParser(strict=False,
-                                               empty_lines_in_values=False,
-                                               dict_type=MultiOrderedDict,
-                                               interpolation=None)
-            config.read([pipewire_socket_unit])
-            socket_names = config.get('Socket', 'ListenStream')
-        else:
-            # The pipewire.socket file was not found, use the default
-            # value used by pipewire as a fallback (c.f. 'man
-            # pipewire.conf'):
-            socket_names = ['%t/pipewire-0']
+
+        config = configparser.ConfigParser(strict=False,
+                                           empty_lines_in_values=False,
+                                           dict_type=MultiOrderedDict,
+                                           interpolation=None)
+        res = config.read([pipewire_socket_unit])
+
         runtime_dir = os.environ['XDG_RUNTIME_DIR']
         return [socket_name.replace('%t', runtime_dir)
-                for socket_name in socket_names]
+                for socket_name in config.get('Socket', 'ListenStream')]
 
     @classmethod
     def enable_pipewire_sockets(klass):
@@ -370,7 +231,9 @@ ret = logind_helpers.open_file_direct(major, minor)
             klass.poll_thread.join()
 
 
-def wrap_call(args, wrapper, extra_env):
+def run_test(args, extra_env):
+    print('Running test case...', file=sys.stderr)
+
     env = {}
     env.update(os.environ)
     env['NO_AT_BRIDGE'] = '1'
@@ -378,10 +241,13 @@ def wrap_call(args, wrapper, extra_env):
     env['GSETTINGS_BACKEND'] = 'memory'
     env['XDG_CURRENT_DESKTOP'] = ''
     env['META_DBUS_RUNNER_ACTIVE'] = '1'
+    env.pop('XDG_SESSION_ID', None)
 
     if extra_env:
         env |= extra_env
 
+    wrapper = os.getenv('META_DBUS_RUNNER_WRAPPER')
+
     if wrapper == 'gdb':
         args = ['gdb', '-ex', 'r', '-ex', 'bt full', '--args'] + args
     elif wrapper == 'rr':
@@ -394,11 +260,26 @@ def wrap_call(args, wrapper, extra_env):
     return p.wait()
 
 
+def run_test_mocked(klass, rest, launch=[], bind_sockets=False, extra_env=None):
+    result = 1
+
+    klass.setUpClass(launch=launch,
+                     bind_sockets=bind_sockets)
+    runner = klass()
+    runner.assertGreater(len(rest), 0)
+
+    try:
+        result = run_test(rest, extra_env)
+    finally:
+        MutterDBusRunner.tearDownClass()
+
+    return result
+
+
 def meta_run(klass, extra_env=None, setup_argparse=None, handle_argparse=None):
     DBusGMainLoop(set_as_default=True)
 
     parser = argparse.ArgumentParser()
-    parser.add_argument('--kvm', action='store_true', default=False)
     parser.add_argument('--launch', action='append', default=[])
     parser.add_argument('--no-isolate-dirs', action='store_true', default=False)
     if setup_argparse:
@@ -416,20 +297,13 @@ def meta_run(klass, extra_env=None, setup_argparse=None, handle_argparse=None):
     else:
         print('WARNING: Command or separator `--` not found', file=sys.stderr)
 
-    if args.no_isolate_dirs:
-        return meta_run_klass(klass, rest,
-                              enable_kvm=args.kvm,
-                              extra_env=extra_env)
-
-    test_root = os.getenv('MUTTER_DBUS_RUNNER_TEST_ROOT')
-    if test_root:
-        print('Reusing MUTTER_DBUS_RUNNER_TEST_ROOT', test_root, file=sys.stderr)
-        return meta_run_klass(klass, rest,
-                              extra_env=extra_env)
+    if os.getenv('META_DBUS_RUNNER_ACTIVE') != None:
+        print('Not re-creating mocked environment...', file=sys.stderr)
+        return run_test(rest, extra_env)
 
     with tempfile.TemporaryDirectory(prefix='mutter-testroot-',
                                      ignore_cleanup_errors=True) as test_root:
-        env_dirs = [
+        env_dirs = [] if args.no_isolate_dirs else [
             'HOME',
             'TMPDIR',
             'XDG_CACHE_HOME',
@@ -437,44 +311,13 @@ def meta_run(klass, extra_env=None, setup_argparse=None, handle_argparse=None):
             'XDG_DATA_HOME',
             'XDG_RUNTIME_DIR',
         ]
-        os.environ['MUTTER_DBUS_RUNNER_TEST_ROOT'] = test_root
-        print('Setup MUTTER_DBUS_RUNNER_TEST_ROOT as', test_root, file=sys.stderr)
         for env_dir in env_dirs:
             directory = os.path.join(test_root, env_dir.lower())
             os.mkdir(directory, mode=0o700)
             os.environ[env_dir] = directory
             print('Setup', env_dir, 'as', directory, file=sys.stderr)
 
-        return meta_run_klass(klass, rest,
-                              enable_kvm=args.kvm,
-                              launch=args.launch,
-                              bind_sockets=True,
-                              extra_env=extra_env)
-
-def meta_run_klass(klass, rest, enable_kvm=False, launch=[], bind_sockets=False, extra_env=None):
-    result = 1
-
-    if os.getenv('META_DBUS_RUNNER_ACTIVE') == None:
-        klass.setUpClass(enable_kvm=enable_kvm,
-                         launch=launch,
-                         bind_sockets=bind_sockets)
-        runner = klass()
-        runner.assertGreater(len(rest), 0)
-        wrapper = os.getenv('META_DBUS_RUNNER_WRAPPER')
-
-        try:
-            print('Running test case...', file=sys.stderr)
-            result = wrap_call(rest, wrapper, extra_env)
-        finally:
-            MutterDBusRunner.tearDownClass()
-    else:
-        try:
-            print(('Inside a nested meta-dbus-runner: '
-                   'Not re-creating mocked environment.'),
-                  file=sys.stderr)
-            print('Running test case...', file=sys.stderr)
-            result = wrap_call(rest, None, extra_env)
-        finally:
-            pass
-
-    return result
+        return run_test_mocked(klass, rest,
+                               launch=args.launch,
+                               bind_sockets=True,
+                               extra_env=extra_env)
