@@ -21,12 +21,13 @@
 
 #include "backends/meta-logical-monitor.h"
 #include "backends/meta-monitor-manager-private.h"
+#include "backends/meta-udev.h"
 #include "backends/meta-virtual-monitor.h"
 #include "backends/native/meta-backend-native.h"
-#include "backends/native/meta-udev.h"
 #include "meta-test/meta-context-test.h"
 #include "tests/drm-mock/drm-mock.h"
 #include "tests/meta-test-utils.h"
+#include "tests/meta-kms-test-utils.h"
 
 typedef enum _State
 {
@@ -136,22 +137,16 @@ meta_test_disconnect_connect (void)
   MetaMonitorManager *monitor_manager =
     meta_backend_get_monitor_manager (backend);
   ClutterActor *stage = meta_backend_get_stage (backend);
-  MetaUdev *udev = meta_backend_native_get_udev (META_BACKEND_NATIVE (backend));
-  g_autolist (GObject) udev_devices = NULL;
-  GUdevDevice *udev_device;
+  MetaUdev *udev = meta_backend_get_udev (backend);
+  g_autoptr (GUdevDevice) udev_device = NULL;
   GList *logical_monitors;
   gulong after_paint_handler_id;
   gulong presented_handler_id;
   ClutterStageView *view;
   CoglFramebuffer *onscreen;
-  g_autoptr (GError) error = NULL;
   State state;
 
-  udev_devices = meta_udev_list_drm_devices (udev,
-                                             META_UDEV_DEVICE_TYPE_CARD,
-                                             &error);
-  g_assert_cmpuint (g_list_length (udev_devices), ==, 1);
-  udev_device = g_list_first (udev_devices)->data;
+  udev_device = meta_get_test_udev_device (udev);
 
   logical_monitors =
     meta_monitor_manager_get_logical_monitors (monitor_manager);
@@ -371,21 +366,15 @@ static void
 emulate_hotplug (void)
 {
   MetaBackend *backend = meta_context_get_backend (test_context);
-  MetaUdev *udev = meta_backend_native_get_udev (META_BACKEND_NATIVE (backend));
-  g_autoptr (GError) error = NULL;
-  g_autolist (GObject) udev_devices = NULL;
-  GUdevDevice *udev_device;
+  MetaUdev *udev = meta_backend_get_udev (backend);
+  g_autoptr (GUdevDevice) udev_device = NULL;
 
-  udev_devices = meta_udev_list_drm_devices (udev,
-                                             META_UDEV_DEVICE_TYPE_CARD,
-                                             &error);
-  g_assert_cmpuint (g_list_length (udev_devices), ==, 1);
-  udev_device = g_list_first (udev_devices)->data;
+  udev_device = meta_get_test_udev_device (udev);
   g_signal_emit_by_name (udev, "hotplug", udev_device);
 }
 
 static void
-meta_test_power_save_implicit_on (void)
+meta_test_power_save_no_implicit_on (void)
 {
   MetaBackend *backend = meta_context_get_backend (test_context);
   MetaMonitorManager *monitor_manager =
@@ -423,9 +412,11 @@ meta_test_power_save_implicit_on (void)
                                 disconnect_connector_filter, NULL);
   emulate_hotplug ();
 
-  while (!power_save_mode_changed || !monitors_changed)
+  while (!monitors_changed)
     g_main_context_iteration (NULL, TRUE);
 
+  g_assert_false (power_save_mode_changed);
+
   g_signal_handler_disconnect (monitor_manager, power_save_handler_id);
   g_signal_handler_disconnect (monitor_manager, monitors_changed_handler_id);
 
@@ -445,8 +436,8 @@ init_tests (void)
                    meta_test_disconnect_connect);
   g_test_add_func ("/hotplug/switch-config",
                    meta_test_switch_config);
-  g_test_add_func ("/hotplug/power-save-implicit-off",
-                   meta_test_power_save_implicit_on);
+  g_test_add_func ("/hotplug/power-save-no-implicit-on",
+                   meta_test_power_save_no_implicit_on);
 }
 
 int
@@ -456,7 +447,7 @@ main (int argc, char *argv[])
 
   context = meta_create_test_context (META_CONTEXT_TEST_TYPE_VKMS,
                                       META_CONTEXT_TEST_FLAG_NO_X11);
-  g_assert (meta_context_configure (context, &argc, &argv, NULL));
+  g_assert_true (meta_context_configure (context, &argc, &argv, NULL));
 
   init_tests ();
 
