@@ -1,6 +1,6 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
-set -e
+set -ex
 
 MUTTER="$1"
 
@@ -11,6 +11,7 @@ fi
 
 export GDK_BACKEND=x11
 export G_DEBUG=fatal-warnings
+export MUTTER_DEBUG=${MUTTER_DEBUG:-x11}
 
 echo \# Launching mutter > /dev/stderr
 $MUTTER --x11 --mutter-plugin="$MUTTER_TEST_PLUGIN_PATH" &
@@ -21,19 +22,22 @@ echo \# Launched with pid $MUTTER1_PID
 sleep 2
 
 echo Launching a couple of X11 clients > /dev/stderr
-zenity --warning &
+${ZENITY:-zenity} --warning &
 ZENITY1_PID=$!
 sleep 2
-zenity --info &
+${ZENITY:-zenity} --info &
 ZENITY2_PID=$!
 sleep 4
 
 echo \# Replacing existing mutter with a new instance > /dev/stderr
 $MUTTER --x11 --replace --mutter-plugin="$MUTTER_TEST_PLUGIN_PATH" &
-echo \# Launched with pid $MUTTER2_PID
 MUTTER2_PID=$!
+echo \# Launched with pid $MUTTER2_PID
 wait $MUTTER1_PID
 
+echo \# Waiting for the second mutter to finish loading
+gdbus wait --session org.gnome.Mutter.IdleMonitor
+
 sleep 2
 
 echo \# Terminating clients > /dev/stderr
