@@ -22,7 +22,6 @@
 
 #pragma once
 
-#include <cairo.h>
 #include <wayland-server.h>
 
 #include "cogl/cogl.h"
@@ -80,6 +79,8 @@ struct _MetaWaylandBuffer
   } single_pixel;
 
   GHashTable *tainted_scanout_onscreens;
+
+  GPtrArray *release_points;
 };
 
 #define META_TYPE_WAYLAND_BUFFER (meta_wayland_buffer_get_type ())
@@ -100,8 +101,10 @@ void                    meta_wayland_buffer_dec_use_count       (MetaWaylandBuff
 gboolean                meta_wayland_buffer_is_y_inverted       (MetaWaylandBuffer     *buffer);
 void                    meta_wayland_buffer_process_damage      (MetaWaylandBuffer     *buffer,
                                                                  MetaMultiTexture      *texture,
-                                                                 cairo_region_t        *region);
+                                                                 MtkRegion             *region);
 CoglScanout *           meta_wayland_buffer_try_acquire_scanout (MetaWaylandBuffer     *buffer,
-                                                                 CoglOnscreen          *onscreen);
+                                                                 CoglOnscreen          *onscreen,
+                                                                 const graphene_rect_t *src_rect,
+                                                                 const MtkRectangle    *dst_rect);
 
 void meta_wayland_init_shm (MetaWaylandCompositor *compositor);
