@@ -36,13 +36,20 @@
 
 #include "backends/meta-dnd-private.h"
 #include "compositor/meta-dnd-actor-private.h"
+#include "compositor/meta-surface-actor.h"
+#include "compositor/meta-window-drag.h"
 #include "core/meta-selection-private.h"
+#include "meta/meta-debug.h"
 #include "meta/meta-selection-source-memory.h"
+#include "meta/meta-wayland-surface.h"
 #include "wayland/meta-selection-source-wayland-private.h"
+#include "wayland/meta-wayland-data-source.h"
 #include "wayland/meta-wayland-dnd-surface.h"
 #include "wayland/meta-wayland-pointer.h"
 #include "wayland/meta-wayland-private.h"
 #include "wayland/meta-wayland-seat.h"
+#include "wayland/meta-wayland-toplevel-drag.h"
+#include "wayland/meta-wayland-types.h"
 
 #define ROOTWINDOW_DROP_MIME "application/x-rootwindow-drop"
 
@@ -211,21 +218,88 @@ on_drag_focus_destroyed (MetaWaylandSurface  *surface,
   grab->drag_focus = NULL;
 }
 
+static void
+meta_wayland_drag_grab_set_cursor (MetaWaylandDragGrab *drag_grab,
+                                   MetaCursor           cursor)
+{
+  MetaWaylandCompositor *compositor =
+    meta_wayland_seat_get_compositor (drag_grab->seat);
+  MetaContext *context = meta_wayland_compositor_get_context (compositor);
+  MetaBackend *backend = meta_context_get_backend (context);
+  MetaCursorTracker *cursor_tracker =
+    meta_backend_get_cursor_tracker (backend);
+  g_autoptr (MetaCursorSprite) cursor_sprite = NULL;
+  MetaCursorRenderer *cursor_renderer;
+
+  cursor_sprite =
+    META_CURSOR_SPRITE (meta_cursor_sprite_xcursor_new (cursor, cursor_tracker));
+  cursor_renderer =
+    meta_backend_get_cursor_renderer_for_device (backend, drag_grab->device);
+
+  if (cursor_renderer && cursor_sprite)
+    meta_cursor_renderer_set_cursor (cursor_renderer, cursor_sprite);
+}
+
+static void
+meta_wayland_drag_grab_update_cursor (MetaWaylandDragGrab *drag_grab)
+{
+  enum wl_data_device_manager_dnd_action action =
+    meta_wayland_data_source_get_current_action (drag_grab->drag_data_source);
+  MetaCursor cursor = META_CURSOR_DEFAULT;
+
+  switch (action)
+    {
+    case WL_DATA_DEVICE_MANAGER_DND_ACTION_NONE:
+      cursor = META_CURSOR_NOT_ALLOWED;
+      break;
+    case WL_DATA_DEVICE_MANAGER_DND_ACTION_MOVE:
+      cursor = META_CURSOR_MOVE;
+      break;
+    case WL_DATA_DEVICE_MANAGER_DND_ACTION_COPY:
+      cursor = META_CURSOR_COPY;
+      break;
+    case WL_DATA_DEVICE_MANAGER_DND_ACTION_ASK:
+      cursor = META_CURSOR_DND_ASK;
+      break;
+    default:
+      break;
+    }
+
+  meta_wayland_drag_grab_set_cursor (drag_grab, cursor);
+}
+
+static void
+on_data_source_action_changed (MetaWaylandDataSource *source,
+                               MetaWaylandDragGrab   *drag_grab)
+{
+  meta_wayland_drag_grab_update_cursor (drag_grab);
+}
+
 static void
 meta_wayland_drag_grab_set_source (MetaWaylandDragGrab   *drag_grab,
                                    MetaWaylandDataSource *source)
 {
   if (drag_grab->drag_data_source)
-    g_object_weak_unref (G_OBJECT (drag_grab->drag_data_source),
-                         drag_grab_data_source_destroyed,
-                         drag_grab);
+    {
+      g_signal_handlers_disconnect_by_func (drag_grab->drag_data_source,
+                                            on_data_source_action_changed,
+                                            drag_grab);
+      g_object_weak_unref (G_OBJECT (drag_grab->drag_data_source),
+                           drag_grab_data_source_destroyed,
+                           drag_grab);
+    }
 
   drag_grab->drag_data_source = source;
 
   if (source)
-    g_object_weak_ref (G_OBJECT (source),
-                       drag_grab_data_source_destroyed,
-                       drag_grab);
+    {
+      g_signal_connect (drag_grab->drag_data_source, "action-changed",
+                        G_CALLBACK (on_data_source_action_changed),
+                        drag_grab);
+      g_object_weak_ref (G_OBJECT (source),
+                         drag_grab_data_source_destroyed,
+                         drag_grab);
+    }
 }
 
 static void
@@ -343,6 +417,12 @@ meta_wayland_drag_grab_get_origin (MetaWaylandDragGrab *drag_grab)
   return drag_grab->drag_origin;
 }
 
+MetaWaylandDataSource *
+meta_wayland_drag_grab_get_data_source (MetaWaylandDragGrab *drag_grab)
+{
+  return drag_grab->drag_data_source;
+}
+
 static void
 data_source_update_user_dnd_action (MetaWaylandDataSource *source,
                                     ClutterModifierType    modifiers)
@@ -398,6 +478,7 @@ data_device_end_drag_grab (MetaWaylandDragGrab *drag_grab)
       drag_grab->handler = NULL;
     }
 
+  meta_wayland_drag_grab_set_cursor (drag_grab, META_CURSOR_DEFAULT);
   meta_dnd_wayland_handle_end_modal (compositor);
 
   g_free (drag_grab);
@@ -503,6 +584,12 @@ data_device_update_position (MetaWaylandDragGrab *drag_grab,
   meta_wayland_surface_set_main_monitor (drag_grab->drag_surface, monitor);
 }
 
+static gboolean
+is_dragging_window (MetaWaylandSeat *seat)
+{
+  return meta_wayland_data_device_get_toplevel_drag (&seat->data_device) != NULL;
+}
+
 static gboolean
 drag_grab_motion (MetaWaylandEventHandler *handler,
 		  const ClutterEvent      *event,
@@ -533,7 +620,7 @@ drag_grab_motion (MetaWaylandEventHandler *handler,
 
   meta_dnd_wayland_on_motion_event (meta_backend_get_dnd (backend), event);
 
-  return CLUTTER_EVENT_STOP;
+  return !is_dragging_window (drag_grab->seat);
 }
 
 static gboolean
@@ -544,6 +631,7 @@ drag_grab_release (MetaWaylandEventHandler *handler,
   MetaWaylandDragGrab *drag_grab = user_data;
   MetaWaylandSeat *seat = drag_grab->seat;
   MetaWaylandDataSource *source = drag_grab->drag_data_source;
+  MetaWaylandToplevelDrag *toplevel_drag;
   gboolean success;
 
   if (drag_grab->device != clutter_event_get_device (event) ||
@@ -558,12 +646,22 @@ drag_grab_release (MetaWaylandEventHandler *handler,
                            CLUTTER_BUTTON5_MASK)) > 1)
     return CLUTTER_EVENT_STOP;
 
+  toplevel_drag = meta_wayland_data_device_get_toplevel_drag (&seat->data_device);
+  if (toplevel_drag)
+    {
+      meta_topic (META_DEBUG_WAYLAND, "Will end xdg_toplevel_drag#%u.",
+                  wl_resource_get_id (toplevel_drag->resource));
+      meta_wayland_data_source_notify_drop_performed (source);
+      meta_wayland_toplevel_drag_end (toplevel_drag);
+    }
+
   if (drag_grab->drag_focus && source &&
       meta_wayland_data_source_has_target (source) &&
       meta_wayland_data_source_get_current_action (source))
     {
       meta_wayland_surface_drag_dest_drop (drag_grab->drag_focus);
-      meta_wayland_data_source_notify_drop_performed (source);
+      if (!meta_wayland_data_source_get_drop_performed (source))
+        meta_wayland_data_source_notify_drop_performed (source);
 
       meta_wayland_data_source_update_in_ask (source);
       success = TRUE;
@@ -602,11 +700,20 @@ drag_grab_key (MetaWaylandEventHandler *handler,
                const ClutterEvent      *event,
                gpointer                 user_data)
 {
+  MetaWaylandToplevelDrag *toplevel_drag;
   MetaWaylandDragGrab *drag_grab = user_data;
   ClutterModifierType modifiers;
 
   if (clutter_event_get_key_symbol (event) == CLUTTER_KEY_Escape)
     {
+      toplevel_drag = meta_wayland_data_device_get_toplevel_drag (&drag_grab->seat->data_device);
+      if (toplevel_drag)
+        {
+          meta_topic (META_DEBUG_WAYLAND, "Will cancel xdg_toplevel_drag#%u.",
+                      wl_resource_get_id (toplevel_drag->resource));
+          meta_wayland_toplevel_drag_end (toplevel_drag);
+        }
+
       meta_wayland_data_device_set_dnd_source (&drag_grab->seat->data_device,
                                                NULL);
       unset_selection_source (&drag_grab->seat->data_device, META_SELECTION_DND);
@@ -1208,6 +1315,8 @@ meta_wayland_data_device_manager_init (MetaWaylandCompositor *compositor)
 			META_WL_DATA_DEVICE_MANAGER_VERSION,
 			compositor, bind_manager) == NULL)
     g_error ("Could not create data_device");
+
+  meta_wayland_init_toplevel_drag (compositor);
 }
 
 void
@@ -1302,3 +1411,12 @@ meta_wayland_data_device_unset_dnd_selection (MetaWaylandDataDevice *data_device
 {
   unset_selection_source (data_device, META_SELECTION_DND);
 }
+
+MetaWaylandToplevelDrag *
+meta_wayland_data_device_get_toplevel_drag (MetaWaylandDataDevice *data_device)
+{
+  if (!data_device->current_grab || !data_device->current_grab->drag_data_source)
+    return NULL;
+
+  return meta_wayland_data_source_get_toplevel_drag (data_device->current_grab->drag_data_source);
+}
