@@ -110,3 +110,10 @@ gboolean meta_wayland_data_source_get_drop_performed (MetaWaylandDataSource *sou
 
 void meta_wayland_data_source_notify_drop_performed (MetaWaylandDataSource *source);
 void meta_wayland_data_source_notify_finish (MetaWaylandDataSource *source);
+
+void
+meta_wayland_data_source_set_toplevel_drag (MetaWaylandDataSource   *source,
+                                            MetaWaylandToplevelDrag *toplevel_drag);
+
+MetaWaylandToplevelDrag *
+meta_wayland_data_source_get_toplevel_drag (MetaWaylandDataSource *source);
