@@ -29,7 +29,6 @@
 #include "wayland/meta-wayland-types.h"
 
 MetaWaylandTabletTool * meta_wayland_tablet_tool_new  (MetaWaylandTabletSeat  *seat,
-                                                       ClutterInputDevice     *device,
                                                        ClutterInputDeviceTool *device_tool);
 void                    meta_wayland_tablet_tool_free (MetaWaylandTabletTool  *tool);
 
@@ -65,3 +64,10 @@ MetaWaylandSurface * meta_wayland_tablet_tool_get_current_surface (MetaWaylandTa
 
 void meta_wayland_tablet_tool_focus_surface (MetaWaylandTabletTool *tool,
                                              MetaWaylandSurface    *surface);
+
+void meta_wayland_tablet_tool_set_cursor_shape (MetaWaylandTabletTool *tool,
+                                                MetaCursor             shape);
+
+gboolean meta_wayland_tablet_tool_check_focus_serial (MetaWaylandTabletTool *tool,
+                                                      struct wl_client      *client,
+                                                      uint32_t               serial);
